/*
 * Suspender.java
 * 
 * An interface which abstracts a suspension in code, which interrupts
 * a thread until a specific condition is satisfied, and then continues
 * normally.
 *
 * Created on March 3, 2007, 11:08 PM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */
package wowiupdater;
/**
 * An interface which abstracts a suspension in code, which interrupts
 * a thread until a specific condition is satisfied, and then continues
 * normally.
 *
 * @author m.delbuono
 */
import javax.swing.*;
import java.awt.*;
public interface Suspender {
    /**
     * Attempts to resolve the issue resulting in the supension. Called
     * when the "resolve" button is clicked. Returns true if resolution was
     * successful, false if it wasn't. Suspend() uses this return value to 
     * determine if suspension should end.
     *
     * @return boolean whether the resolution was successful
     */
    //protected boolean Resolve();
    
    /**
     * Suspends the current thread until resolution by the user occurs.
     * Creates a button in the location specified by GridBagConstraints c,
     * in panel p which calls Resolve() on click. Does not return until
     * resolution is successful. This should ONLY be used in daemon threads
     * or this could result in a lockdown of the JVM.
     *
     * @param c the constraints indicating the position of the button
     * @param p the panel to apply the button to
     * @param b the progress bar related to this suspension
     * @throws java.util.concurrent.CancellationException if the operation
     * was cancelled
     * @throws java.lang.InterruptedException if the thread was interrupted
     * while in a wait loop
     */
    public void Suspend(GridBagConstraints c, JPanel p, JProgressBar b) 
        throws java.util.concurrent.CancellationException,
                java.lang.InterruptedException;
    
}
