/*
 * AddonInstallerDialog.java
 *
 * Allows the user to see what favorites appear to be missing, allows
 * to install those favorites, and allows to specify a UID manually
 * for an interface to install. This is unlike the update process as
 * there is nothing to backup or check beforehand. Just a straight install.
 *
 * Created on April 2, 2007, 7:06 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.util.Vector;
import java.util.concurrent.Callable;
import javax.swing.JList;
import javax.swing.JOptionPane;

/**
 * Allows the user to see what favorites appear to be missing, allows
 * to install those favorites, and allows to specify a UID manually
 * for an interface to install. This is unlike the update process as
 * there is nothing to backup or check beforehand. Just a straight install.
 *
 * @author  m.delbuono
 */
public class AddonInstallerDialog extends javax.swing.JDialog 
        implements Callable
{
    private Addon a;
    private Thread t;
    private JList list;
    //private Vector<Addon> listData;
    
    /** Identifiers specifying what type of install was selected */
    public static final int
            ACTION_NONE = 0,
            ACTION_SELECTED = 1,
            ACTION_FAVORITES = 2;

    private Vector<Addon> listData;
    /** Status code for the dialog box */
    public int code = ACTION_NONE;
    /** Creates new form AddonInstallerDialog */
    public AddonInstallerDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        buttonGroup1 = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        scrollPane = new javax.swing.JScrollPane();
        jPanel2 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        idField = new javax.swing.JTextField();
        nameField = new javax.swing.JTextField();
        authorField = new javax.swing.JTextField();
        pb = new javax.swing.JProgressBar();
        installAll = new javax.swing.JToggleButton();
        install = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Install Manager");
        jLabel1.setText("Please select the addons you wish to install");

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Favorites List"));
        list = new JList();
        list.setListData(new Vector<Addon>());
        scrollPane.setViewportView(list);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(scrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 169, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, scrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 133, Short.MAX_VALUE)
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Addon Information"));
        jLabel2.setText("ID");

        jLabel3.setText("Name");

        jLabel4.setText("Author");

        idField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                idFieldActionPerformed(evt);
            }
        });
        idField.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                idFieldInputMethodTextChanged(evt);
            }
        });
        idField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                idFieldKeyTyped(evt);
            }
        });

        nameField.setEditable(false);

        authorField.setEditable(false);

        pb.setMinimum(0);
        pb.setMaximum(1);
        pb.setValue(0);

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, pb, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 198, Short.MAX_VALUE)
                    .add(jPanel2Layout.createSequentialGroup()
                        .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel2)
                            .add(jLabel3)
                            .add(jLabel4))
                        .add(23, 23, 23)
                        .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(nameField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 142, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(idField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 142, Short.MAX_VALUE)
                            .add(authorField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 142, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(idField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(nameField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel4)
                    .add(authorField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(pb, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );

        installAll.setText("Install All Favorites");
        installAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                installAllActionPerformed(evt);
            }
        });

        install.setText("Install Selected");
        install.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                installActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(layout.createSequentialGroup()
                                .add(install)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(installAll))
                            .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jLabel1))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(11, 11, 11)
                .add(jLabel1)
                .add(10, 10, 10)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(installAll)
                            .add(install)))
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void installAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_installAllActionPerformed
        code = ACTION_FAVORITES;
        this.dispose();
    }//GEN-LAST:event_installAllActionPerformed

    private void installActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_installActionPerformed
        code = ACTION_SELECTED;
        this.dispose();
    }//GEN-LAST:event_installActionPerformed

    private void idFieldInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_idFieldInputMethodTextChanged
        JOptionPane.showMessageDialog(null, "OK");
    }//GEN-LAST:event_idFieldInputMethodTextChanged

    private void idFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_idFieldActionPerformed
        JOptionPane.showMessageDialog(null, "OK");
    }//GEN-LAST:event_idFieldActionPerformed

    private void idFieldKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_idFieldKeyTyped
        // Update info
        if (t != null && t.isAlive())
        {
            FileReporterHandler.verbose("T!! "+Thread.currentThread().getName()
                +" sends interrupt to thread "+t.getName());
            t.interrupt();
        }
        pb.setIndeterminate(true);
        
        t = null;
        a = null;
        nameField.setText("");
        authorField.setText("");
        
        t = new Thread(a = new Addon(this, this.idField),  
                "Installer"+System.currentTimeMillis());
        FileReporterHandler.verbose("T++ "+Thread.currentThread().getName()
            +" creates thread "+t.getName());
        t.start();
       
            
    }//GEN-LAST:event_idFieldKeyTyped
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new AddonInstallerDialog(new javax.swing.JFrame(), true).setVisible(true);
            }
        });
    }

    /**
     * A callback function to use when finished processing a request issued
     * by this dialog.<br /><br />
     * <b>Note:</b> Always returns <tt>null</tt>. Use of <tt>Object</tt> is 
     * a limitation of Callable
     * @return <tt>null</tt>
     */
    public Object call()
    {
        this.nameField.setText(a.name);
        this.authorField.setText(a.author);
        pb.setIndeterminate(false);
        return null;
    }
    

    /** Adds addons to the list
     * @param list the addon list to set this list too
     */
    public void set(Vector<Addon> list)
    {
        //this.listData = list;
        this.list.setListData(list);
        this.listData = list;
        repaint();
    }
    
    /** Retrieves the listing of addons selected
     * @return a Vector of Addons indicating which addons were selected
     */
    public Vector<Addon> selected()
    {
        int[] l = (list.getSelectedIndices());
        Vector<Addon> v = new Vector<Addon>();
        for (int i = 0; i < l.length; i++)
            v.add(listData.get(l[i]));
        return v;
    }
    
    /** Retrives all addons that need to be installed. This may differ
     * from selected() if ACTION_FAVORITES is currently in use.
     * @return a Vector of Addons that need to be installed
     * @see wowiupdater.AddonInstallerDialog#selected()
     */
    public Vector<Addon> get()
    {
        Vector<Addon> list = new Vector<Addon>();
        if (code == AddonInstallerDialog.ACTION_SELECTED)
        {
            Vector<Addon> thisList = selected();
            for (int i = 0; i < thisList.size(); i++)
                list.add(thisList.get(i));
            
            // Add in the manual one if needed
            if (a != null && a.name.compareToIgnoreCase("") != 0) 
                list.add(a);                            // Add it
        }
        else if (code == AddonInstallerDialog.ACTION_FAVORITES)
        {
            // Add all favorites
            // First we mark them all selected
            this.list.setSelectionInterval(0, listData.size()-1);
          
            // Add them, ignore the manual entry
            Vector<Addon> thisList = selected();
            for (int i = 0; i < thisList.size(); i++)
                list.add(thisList.get(i));
            
        }
        // ACTION_NONE (or anything else) will produce an empty list
        return list;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField authorField;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JTextField idField;
    private javax.swing.JButton install;
    private javax.swing.JToggleButton installAll;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JTextField nameField;
    private javax.swing.JProgressBar pb;
    private javax.swing.JScrollPane scrollPane;
    // End of variables declaration//GEN-END:variables
    
    
}
