/*
 * BackupEntry.java
 *
 * A single entry of a backup manager. This stores critical information about
 * a stored backup, such as addon, reason, and a timestamp.
 *
 * Created on April 6, 2007, 5:56 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 *
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.io.File;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * A single entry of a backup manager. This stores critical information about
 * a stored backup, such as addon, reason, and a timestamp.
 *
 * @author m.delbuono
 */
public class BackupEntry {
    public long timestamp;
    public File location;
    public int uid;
    public String name;
    public String reasoning;
    
    /** Creates a new instance of BackupEntry */
    public BackupEntry(long timestamp, File location,
            int uid, String name, String reasoning) 
    {
        this.timestamp = timestamp;
        this.location = location;
        this.uid = uid;
        this.name = name;
        this.reasoning = reasoning;
    }
    
    /** Represents this <tt>BackupEntry</tt> as a String. Used for list controls 
     * @return the String representation of this <tt>BackupEntry</tt>
     */
    public String toString()
    {
        Date time = new Date(timestamp);
        SimpleDateFormat fmt = new SimpleDateFormat(
                //  2 Jul 2007 1:06 PM
                   "d MMM yyyy h:mm a"
                );
        String retval = fmt.format(time);
        retval += ": " + (name==null?reasoning:(name + " - " + reasoning));
        return retval;
    }
    
}
