/*
 * DownloadConfirmationSuspender.java
 *
 * An implementation of the Suspender class which suspends a thread
 * while waiting for a user to confirm a download.
 *
 * Created on March 4, 2007, 5:02 PM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.CancellationException;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import java.awt.*;

/**
 * An implementation of the Suspender class which suspends a thread
 * while waiting for a user to confirm a download.
 * @author m.delbuono
 */
public class DownloadConfirmationSuspender extends ResolverSuspender {
    volatile boolean isResolved = false;
    volatile boolean okToInstall = false;
    
    /** Determines if the user stated it was OK to install this download
     * @return <tt>true</tt> if the user confirmed the download, <tt>false</tt>
     * otherwise
     */
    public boolean isOkToInstall()
    {
        return okToInstall;
    }
    
    Addon addon;
    /** Creates a new instance of DownloadConfirmationSuspender */
    public DownloadConfirmationSuspender(Addon a) {
        addon = a;
    }

    private synchronized void Confirm() {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" locks DownloadConfirmationSuspender.Confirm()");
        // User wants to confirm, so let's show the dialog and confirm it
        DownloadConfirmationFrame f = new DownloadConfirmationFrame();
        f.load(addon);
        
        f.setVisible(true);
        f.validate();
        f.setLocationRelativeTo(null);
        f.SetHandler(this);
                
        //return (f.choice == DownloadConfirmationFrame.INSTALL_CHOICE);
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" releases DownloadConfirmationSuspender.Confirm()");
    }

    /** Suspends this thread, waiting for user input.
     * @param c The <tt>GridBagConstraints</tt> to handle this suspension
     * @param p the <tt>JPanel</tt> to use for controls
     * @param b the <tt>JProgressBar</tt> to handle showing information
     */
    public synchronized void Suspend(GridBagConstraints c, JPanel p, JProgressBar b) 
        throws CancellationException, InterruptedException {
        
        
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" locks DownloadConfirmationSuspender.Suspend()");
        //throw new UnsupportedOperationException("Not yet implemented");
        constraints = c;
        
        // Replace the progress bar with a button
        this.p = p;
        oldBar = b;
        GenerateResolutionButton("Confirm");
        
        isResolved = false;
        while (!isResolved)         // changes via action listener
        {
            Main.f.repaint();
            FileReporterHandler.verbose("W++ "+Thread.currentThread().getName()+" waits for notification from 'this'");
            this.wait();  // wait for wakeup (notify)
            FileReporterHandler.verbose("W-- "+Thread.currentThread().getName()+" received notify from 'this'");
            
            // Also watch for a possible semaphore flag
            CheckForRebuild();
            
        }
        
        DisposeResolutionButton();
        
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" releases DownloadConfirmationSuspender.Suspend()");
        
    }

    /** Called when an <tt>ActionEvent</tt> occurs
     * @param e The <tt>ActionEvent</tt> that occurred
     */
    public synchronized void actionPerformed(ActionEvent e) {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" locks DownloadConfirmationSuspender.actionPerformed()");
        //throw new UnsupportedOperationException("Not yet implemented");
        if (resolveBtn == e.getSource())
        {
            DisableResolve();
            Confirm();
            FileReporterHandler.verbose("W** "+Thread.currentThread().getName()+" broadcasts notify to 'this' (ALL)");
            this.notifyAll();
        }
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" releases DownloadConfirmationSuspender.actionPerformed()");
    }
    
    /** Called when the user has made a choice, and updates the semaphores
     * @param chosen the choice the user made
     */
    public synchronized void choice(int chosen)
    {
        EnableResolve();
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" locks DownloadConfirmationSuspender.choice()");
        isResolved = true;
        okToInstall = (chosen == DownloadConfirmationFrame.INSTALL_CHOICE);
        FileReporterHandler.verbose("W** "+Thread.currentThread().getName()+" broadcasts notify to 'this' (ALL)");
        this.notifyAll();
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" releases DownloadconfirmationSuspender.choice()");
    }
    
}
