/*
 * GetUsernameDialog.java
 *
 * Requests the username of the user, for use in retrieving the favorites 
 * list.
 *
 * Created on March 10, 2007, 9:30 PM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

/**
 * Requests the username of the user, for use in retrieving the favorites 
 * list.
 * @author  m.delbuono
 */
public class GetUsernameDialog extends javax.swing.JDialog {
    
    /** Creates new form GetUsernameDialog */
    public GetUsernameDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        errText.setText("");
        password.setText("");
        this.getRootPane().setDefaultButton(this.jButton2);
    }
    
    /** Creates new form GetUsernameDialog with an error message
     *  and initial data
     *
     * @param <tt>parent</tt> the parent of this form
     * @param <tt>modal</tt> <tt>true</tt> if this dialog is modal, <tt>false</tt>
     * if it is not.
     * @param <tt>err</tt> the error message to display
     * @param <tt>user</tt> the initial username data
     * @param <tt>pass</tt> the inital password data
     */
    public GetUsernameDialog(java.awt.Frame parent, boolean modal, String err,
            String user, String pass)
    {
        super(parent, modal);
        initComponents();
        errText.setText(err);
        jTextField1.setText(user);
        password.setText("");   // since encrypted, don't put the pass anymore
        this.getRootPane().setDefaultButton(this.jButton2);
    }
    
    /** Status codes identifying the dialog's state */
    public static final int
            STATUS_WAIT = 2,
            STATUS_OK = 0,
            STATUS_CANCEL = 1;
    
    private volatile int status = STATUS_WAIT;
    /** Gets the status of this dialog box 
     * @return one of <tt>GetUsernameDialog.STATUS_WAIT,
     * GetUsernameDialog.STATUS_OK, </tt>or<tt> GetUsernameDialog.STATUS_CANCEL
     * </tt>
     */
    public int getStatus()
    {
        return status;
    }
    /** Gets the username supplied by the user in this dialog box 
     * @return the username specified
     */
    public String getUsername()
    {       
        return jTextField1.getText();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        password = new javax.swing.JPasswordField();
        errText = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setModal(true);
        setResizable(false);
        jLabel1.setText("Please enter your login username and password for WoWInterface.");

        jLabel2.setText("and to retrieve your favorites list to assist with detecting interface IDs.");

        jLabel3.setText("Username");

        jTextField1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jTextField1FocusGained(evt);
            }
        });

        jButton1.setText("Cancel");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setText("OK");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jLabel5.setText("the UI Manager uses this information to verify your premium membership");

        jLabel4.setText("Password");

        password.setText("jPasswordField1");
        password.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                passwordFocusGained(evt);
            }
        });

        errText.setForeground(new java.awt.Color(204, 0, 0));
        errText.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        errText.setText("<< Error Text >>");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(errText, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .add(jLabel1)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(jButton2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jButton1))
                    .add(jLabel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel2)
                        .add(layout.createSequentialGroup()
                            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jLabel3)
                                .add(jLabel4))
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 62, Short.MAX_VALUE)
                            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                .add(password)
                                .add(jTextField1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 232, Short.MAX_VALUE)))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(0, 0, 0)
                .add(jLabel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(0, 0, 0)
                .add(jLabel2)
                .add(26, 26, 26)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel4)
                    .add(password, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(11, 11, 11)
                .add(errText)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 12, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jButton1)
                    .add(jButton2))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jTextField1FocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextField1FocusGained
        // Highlight entire contents
        this.jTextField1.setSelectionStart(0);
        this.jTextField1.setSelectionEnd(this.jTextField1.getText().length());
    }//GEN-LAST:event_jTextField1FocusGained

    private void passwordFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_passwordFocusGained
        // Highlight the entire contents to allow easy modification
        this.password.setSelectionStart(0);
        this.password.setSelectionEnd(this.password.getPassword().length);
    }//GEN-LAST:event_passwordFocusGained

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        this.status = STATUS_CANCEL;
        this.dispose();
        
        //FileReporterHandler.verbose("W** "+Thread.currentThread().getName()+" sending notify to 'this' (GetUsernameDialog)");
        //this.notifyAll();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        this.status = STATUS_OK;
        this.dispose();
        
        //FileReporterHandler.verbose("W** "+Thread.currentThread().getName()+" sending notify to 'this' (GetUsernameDialog)");
        //this.notifyAll();
    }//GEN-LAST:event_jButton2ActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new GetUsernameDialog(new javax.swing.JFrame(), true).setVisible(true);
            }
        });
    }

    String getPassword() throws java.security.NoSuchAlgorithmException {    
        byte[] pw;        
        try
        {
            pw = (new String(password.getPassword())).getBytes("8859_1");
        }
        catch (java.io.UnsupportedEncodingException e)
        {
            FileReporterHandler.err("Unable to encode user password in ISO-8859-1. Falling back to default charset.");
            pw = (new String(password.getPassword())).getBytes();
        }
            
        
        // Form an MD5 encryption
        java.security.MessageDigest md = java.security.MessageDigest.getInstance("MD5");
        
        // Generate the digest
        md.update(pw);
        
        // Convert to a string form
        String encrypted = "";
        byte[] digest = md.digest();
        
        // We need to pad with 0s
        for (int n = 16; n > digest.length; n--)
            encrypted += "0";
        
        for (byte b : digest)
        {
            encrypted = encrypted + String.format("%02x", b&0xff);
        }
        FileReporterHandler.verbose("E>> Encrypted password, irreversible hash: "+encrypted);
        return encrypted;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel errText;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JPasswordField password;
    // End of variables declaration//GEN-END:variables
    
}
