/*
 * Main.java
 *
 * WoWIUpdater's Main class. This class handles all startup activity,
 * along with some static globals (for modularity). This includes
 * setup of the UI for default look-and-feel, loading of interface
 * addons, and starting the main thread queue. This class should
 * be executed directly.
 *
 * Created on March 1, 2007, 10:38 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;
import java.awt.Image;
import java.net.HttpURLConnection;
//import java.awt.Toolkit;
import java.util.Vector;
import javax.swing.*;
import java.io.*;
import java.util.Calendar;


/** The main class. This class handles all initialization for the application
 * and starts up the first processes.
 *
 * @author m.delbuono
 */
public class Main implements Runnable {
    // Note these cannot be "final" because they are init in initConstants()
    // just remember not to change them!
    /** The minimum java version required to run */
    private static final float MIN_JAVA_VERSION = 1.5f;
    /** The working directory path. Where to store config files, etc.
     * This is dependent upon OS.
     */
    public static String WD_PATH;
    /** The name of the application configuration file */
    public static String CFG_FILE;
    /** The name of the favorites configuration file */
    public static String FAVS_FILE;
    
    /** The version of the configuration file, used to determine whether
     * or not the config file can be read
     */
    public static String CONFIG_VERSION = "0.2";
        
    /** Initializes the constants which Main uses */
    private static void initConstants() {    
        // Determine OS
        String os = System.getProperty("os.name");
        if (os.contains("Windows"))
            WD_PATH = System.getProperty("user.dir");
        else if (os.contains("Linux"))
            WD_PATH = System.getProperty("user.home")+"/wowiupdater";
        else if (os.contains("Mac OS"))
            WD_PATH = System.getProperty("user.home")+"/Library/com.wowinterface.wowiupdater";
        else
            throw new UnsupportedOperationException("Unknown OS "+System.getProperty("os.name"));
       
        CFG_FILE = WD_PATH + "/config.wic";
        FAVS_FILE = WD_PATH + "/favorites.wic";
        File f = new File(WD_PATH);
       
        if (!f.exists())
            f.mkdirs(); // Make directory if needed
        
        // Validate java version (>= 1.5)
        String version = System.getProperty("java.version");
        version = version.substring(0, 3);  // Only major.minor, nothing further
        float vernum = Float.parseFloat(version);
        if (vernum < MIN_JAVA_VERSION)
        {
            System.out.println("Error: Your current Java Runtime Environment" +
                    " (JRE) version is "+vernum+".");
            System.out.println("Version "+MIN_JAVA_VERSION+" is required to run.");
            System.out.println("Please download the latest JRE from");
            System.out.println("   http://www.java.com/en/download/index.jsp");
            System.out.println("");
            System.out.println("The program will now exit.");
            
            //FileReporterHandler.err("Unable to start. Need JRE "+MIN_JAVA+VERSION+", have "+vernum);
                       
            System.exit(1);            
        }
        
        
        
        
       

    }
    
    /** The name of a data file placed in addons for update data */
    public static String DATA_FILE = "update.wid";
    
    /** The default username to use for WoWI login */
    public static final String UN_DEFAULT = "default";
    
    /** The default password to use for WoWI login */
    public static final String PW_DEFAULT = "default";
    
    /** The string specifying the title of the application */
    public static final String APPTITLE = "WoWInterface UI Manager";
    /** The string specifying the version of this application */
    public static final String VERSION = "0.2.10";
    //public static final int COUNT_PATHS = 5;
    /** An array of default paths to check for the WoW executable. Searched
     * in order listed.
     */
    public static final String[] DEFAULT_PATHS = {
        "/Program Files/World of Warcraft/",
        "/World of Warcraft/",
        "/bin/World of Warcraft/",
        "/sbin/World of Warcraft/",
        "/Applications/World of Warcraft/",
        "/Games/World of Warcraft/",
        
                
    };
    
    public static boolean verboseMode = false, errTestMode = false,
            libTest = false;
    
    /** The URL to connect to */
    public static final String WOWI_URL = "http://www.wowinterface.com/";
    /** A format string specifying the document to download from WOWI_URL */
    public static final String UPDATE_FORMAT = "patcher/info-%d.xml";
    /** A format string specifying the favorites to download from WOWI_URL */
    public static final String FAVS_FORMAT = "patcher/favs/%s/%s.xml";
    /** The location of the backups folder, relative to <tt>Main.wowPath</tt> */
    public static final String REL_BACKUPS = "Backups/"; 
    /** The location of the addons folder, relative to <tt>Main.wowPath</tt> */
    public static final String REL_ADDONS = "Interface/Addons";
    /** The location of the Saved Variables bsae, relative to <tt>Main.wowPath</tt> */
    public static final String REL_SV = "WTF/Account";
    /** The location of the backups manifest, relative to <tt>Main.wowPath</tt> */
    public static final String REL_BACKUPLIST = REL_BACKUPS + "listing.wid";
    /** A folder in the WoW directory that we know will always exist (to test) */
    public static final String REL_EXISTING = "Cache";
    
    /** The maximum UID to accept. This should be large enough for now */
    public static final int MAX_UID = 32767;
    
    /** The user's WoWInterface username */
    public static String Username = null;
    
    /** The user's WoWInterface password */
    public static String Password = null;
    
    /*
    public static final int
            THREAD_DEAD = 0,
            THREAD_ALIVE = 1;
    
    public static int mainThreadStatus = THREAD_DEAD;   // tracks whether we're in main()
     */
    
   
    //private static Thread self = null;
    
    /** A File pointing to the WoW directory */
    protected static File wowPath = null;
    /** The "updater frame," where users register addons */
    protected static UpdaterFrame addonListFrame;
    /** The one and only main frame */
    protected static MainFrame f;
    
    /** Time, in milliseconds, to wait between thread starts */
    public static int THREADLOCK = 100;
    /** Semaphore indicating number of currently suspended threads. */
    public static volatile int suspendedCount = 0;
    
    // Declares which thread has rights to the favorites list
    // (Only one can have access at a time in case changes are made to it)
    // MainFrame's thread manager will handle distributing the lease
    //## We removed this in favor of a single update
    //public static AddonUpdateProcess favoritesLock = null;
    
    /** The favorites list for this application */
    public static FavoritesList favorites = null;
    
    /** The backup manager for this application */
    public static BackupManager backupManager = null; // Init later so we have wowpath
    
    /** Indicates what addons need to be freshly installed */
    public static Vector<Addon> installList = new Vector<Addon>();
    
    /** The WoWInterface icon this application uses */
    public static Image icon;
    
    
    /** Creates a new instance of Main */
    public Main() {
    }
    
    /** The main function. This is where the program execution starts.
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
      
        
        //self = Thread.currentThread();
        initConstants();       
        
        for (int i = 0; i < args.length; i++)
        {
            if (args[i].compareToIgnoreCase("--errtest")==0)
                errTestMode = true;
            else if (args[i].compareToIgnoreCase("--verbose")==0)
                verboseMode = true;
            else if (args[i].compareToIgnoreCase("--libtest") == 0)
                libTest = true;
        }
        
        if (libTest)
        {
            
            System.out.println("Testing load of library...");
            //RARExtractor.test();
            System.out.println("Testing read of test RAR");
            File f = new File("C:\\testrar.rar");
            //RARExtractor e = new RARExtractor(f);
            //e.dump();
            //int[] a = e.get(1);
            ///for (int i = 0; i < a.length; i++)
            //    System.out.print((char)a[i]);
            return;
          
            
        }
        
        
        
       

        
        FileReporterHandler.init(); 
        
        FileReporterHandler.verbose("@@@ "+Thread.currentThread().getName()+" begins application main");

        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
/*        Toolkit t = Toolkit.getDefaultToolkit();
        icon = t.createImage("wowiupdater.ico"); */
        
        
        Runtime.getRuntime().addShutdownHook(new FileReporterHandler());
        
        if (errTestMode)
        {
            FileReporterHandler.err("TRAP -1 > ERROR LOG TEST");
            FileReporterHandler.dump(new Exception("Error Test"));
            System.err.println("TRAP -1 > ERROR LOG TEST: OK!");
            return;
        }
        
        Thread.setDefaultUncaughtExceptionHandler(new FileReporterHandler());
        Thread.currentThread().setUncaughtExceptionHandler(new FileReporterHandler());
        
        Main.icon = new ImageIcon("wowi-icon.gif").getImage();
        
        f = new MainFrame();
        f.SetProgress(-1, -1, -1);
        f.SetStatus("Loading configuration, please wait...");
        f.setVisible(true);
        f.setLocationRelativeTo(null);
        // TODO: Set an icon here
        f.setIconImage(Main.icon);
        f.setTitle(APPTITLE + " v"+VERSION);
                
        // Let's load up an addon list frame too so we can add unknown
        // addons to it
        addonListFrame = new UpdaterFrame();
        addonListFrame.setLocationRelativeTo(null);
        
        loadWowPath();
        if (!wowPath.exists())
            wowPath.mkdirs();       // Mkdirs if needed (maybe it went missing?)
                                    // (Prevents an app stall)
        
        // Now that we have the wow path, make the backup manager
        backupManager = new BackupManager();
        
        //buildFavoritesList();
        buildMainFrame();
/*
        f.SetStatus("Parsing addons...");

        File interfaceDirectory = new File(wowPath, "Interface");
        File directory = new File(interfaceDirectory, "Addons");

        if (!directory.isDirectory())
            throw new FileNotFoundException("Cannot isolate Interface directory.");

        File[] addonDirs = directory.listFiles();
        f.SetProgress(0, 0, addonDirs.length);
        for (int i = 0; i < addonDirs.length; i++)
        {
            f.SetStatus("Scanning addon '"+addonDirs[i]+"'...");
            try
            {
                File thisWID = new File(addonDirs[i], "update.wid");
                Addon a = new Addon(thisWID);   // Get this file
                f.AddAddon(a);            
            }
            catch (IllegalArgumentException e)
            {
                // This indicates that we don't want to touch this one (it's
                // a dependent). Just continue
                continue;
            }
            catch (FileNotFoundException e)
            {
                // That's ok... just skip this one and move to the next
                // Add it to the unknowns list, too
                addonListFrame.AddPath(addonDirs[i]);
            }
            f.SetProgress(i, -1, -1);
            //for (int n = 0; n < 10000000; n++);   // [DEBUG] Just to slow things down
        }

        //f.SetPath(directory.getPath());
        f.SetStatus("Ready");
        f.SetProgress(0,0,1);   // Set progress bar do nothing
        f.validate();



        // Now we've done all the setting up, start executing!
        f.ExecuteUpdate();
 */
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" ends Main.main()");
    }
    
    /** Builds the main frame and starts its process by calling <tt>ExecuteUpdate()</tt>
     *@throws FileNotFoundException if the addons directory could not be found
     *@throws IOException if an I/O error occurred while attempting to build the main frame
     */
    private static void buildMainFrame() throws FileNotFoundException, IOException
    {
        // Grab lock on MainFrame while we rebuild...
        FileReporterHandler.verbose("L?? "+Thread.currentThread().getName()+" requests lock on 'Main.f'");

        synchronized(installList)
        {
            FileReporterHandler.verbose("L++ "+Thread.currentThread().getName()+" obtains lock on 'Main.f'");
            f.SetStatus("Parsing addons...");
            File interfaceDirectory = new File(wowPath, "Interface");
            File directory = new File(interfaceDirectory, "Addons");

            if (!directory.isDirectory())
                throw new FileNotFoundException("Cannot isolate Interface directory.");

            // Look for new addons to install (put them at the top)
            for (int i = 0; i < installList.size(); i++)
            {
                f.AddAddon(installList.get(i));
            }

            // Now clear out that list
            installList.clear();

            // Now process known addons
            File[] addonDirs = directory.listFiles();
            f.SetProgress(0, 0, addonDirs.length);
            for (int i = 0; i < addonDirs.length; i++)
            {
                f.SetStatus("Scanning addon '"+addonDirs[i]+"'...");
                try
                {
                    File thisWID = new File(addonDirs[i], Main.DATA_FILE);
                    Addon a = new Addon(thisWID);   // Get this file
                    f.AddAddon(a);            
                }
                catch (IllegalArgumentException e)
                {
                    // This indicates that we don't want to touch this one (it's
                    // a dependent). Just continue
                    continue;
                }
                catch (FileNotFoundException e)
                {
                    // That's ok... just skip this one and move to the next
                    // Add it to the unknowns list, too
                    addonListFrame.AddPath(addonDirs[i]);
                }
                f.SetProgress(i, -1, -1);
                //for (int n = 0; n < 10000000; n++);   // [DEBUG] Just to slow things down
            }
            FileReporterHandler.verbose("L++ "+Thread.currentThread().getName()+" releases lock on 'Main.f'");



            //f.SetPath(directory.getPath());
            f.SetStatus("Ready");
            f.SetProgress(0,0,1);   // Set progress bar do nothing
            
            //f.notifyAll();
        }
        f.validate();


        // Now we've done all the setting up, start executing!
        try
        {
            f.ExecuteUpdate();
        }
        catch (Throwable t)
        {
            FileReporterHandler.err("System Panic! Uncaught exception in Main.f.ExecuteUpdate()");
            f.setFrameState(MainFrame.PANIC_STATE);
            Thread.getDefaultUncaughtExceptionHandler().uncaughtException(Thread.currentThread(), t);
        }
    }
    
    /** Loads the WoW path into the appropriate static class variable.
     * This first searches default paths, and if not found, asks the user
     * to point to it.
     *
     * @throws IOException if an I/O error occurred during read/write
     */ 
    private static void loadWowPath() throws IOException, java.security.NoSuchAlgorithmException
    {
        // Let's start looking for addons
        // Load our config file. If we don't have a current WoW location, find defaults
        boolean adjustedUN = false;
        java.io.File configFile = new File(CFG_FILE);
        if (configFile.exists())
        {
            FileReader fr = new FileReader(configFile);
            BufferedReader reader = new BufferedReader(fr);

            // Let's read our config file and try to get path/username
            try
            {
                String cfgVer = reader.readLine();
                if (!cfgVer.equals(CONFIG_VERSION))
                {
                    throw new IOException("Invalid config file version "+cfgVer);
                }
                wowPath = new File(reader.readLine());
                Username = reader.readLine();
                Password = reader.readLine();
                
                 
                if (!wowPath.exists())
                    wowPath = null;             // Config file is invalid
            }
            catch (IOException e)
            {
                FileReporterHandler.verbose("!!! Unable to read configuration file: "+e.getMessage());
                // Caught the exception but let's ignore it.. we can try default paths
                wowPath = null;
            }
            catch (NullPointerException e)
            {
                FileReporterHandler.verbose("!!! Unable to read configuration file: "+e.getMessage());
                // Same problem
                wowPath = null;
            }
            finally
            {
                reader.close();
                fr.close();
            }
            
            if (wowPath != null && Username != null && Password != null)
                return;                     // We got what we needed and don't need to write the
                                            // config file; let's return
        }
        
        // if wowPath is not null, we have a path, otherwise let's try defaults
        if (wowPath == null)
        {
            java.io.File currentPath;
            for (int x = 0; x < DEFAULT_PATHS.length; x++)
            {
                // Look for this current path's WoW.exe
                currentPath = new File(DEFAULT_PATHS[x]);
                if (currentPath.exists())
                {
                    wowPath = new File(DEFAULT_PATHS[x]);
                    break;
                }
            }
        }
        
        // Okay... if wowPath STILL is null, we don't have a config file
        // and need to ask user for the path
        while (wowPath == null)
        {
            JFileChooser fc = new JFileChooser();
            fc.setDialogTitle("Locate World of Warcraft Application");
            fc.setDialogType(JFileChooser.SAVE_DIALOG);
            fc.setFileHidingEnabled(false);     // Some OSes will hide; we need this false
            int result = fc.showDialog(null, "Select");
            if (result == JFileChooser.ERROR_OPTION ||
                    result == JFileChooser.CANCEL_OPTION)
            {
                System.exit(1);         // Cancel application - cannot open without WoW
            }
            if (fc.getSelectedFile().exists())
            {
                // Perform one more check - see if we can find the Cache
                // folder here. If not, we'll assume it's bad
                wowPath = fc.getSelectedFile().getParentFile();
                if (!(new File(wowPath, REL_ADDONS)).exists())
                {
                    // See if Cache exists. If it does, make Interface/Addons
                    // if not, it must be bad.
                    if (!(new File(wowPath, REL_EXISTING)).exists())
                        wowPath = null;
                    else
                        (new File(wowPath, REL_ADDONS)).mkdirs();   // Make Interface/Addons
                }
            }
        }
        
        if (Username == null || Password == null)
        {
            GetUsernameDialog nameDlg = new GetUsernameDialog(null, true);
            //FileReporterHandler.verbose("L?? "+Thread.currentThread()+" requests lock on local nameDlg (GetUsernameDialog)");
            // BUGFIX: Events couldn't be processed as main held the lock, so
            // we're removing the synchronized block and making it modal.
            /*synchronized(nameDlg)
            {*/
                // FileReporterHandler.verbose("L++ "+Thread.currentThread()+" obtained lock on local nameDlg (GetUsernameDialog)");
                nameDlg.setLocationRelativeTo(null);    // center
                nameDlg.setVisible(true);
                // Wait for the name to be given
                /*while (nameDlg.getStatus() == GetUsernameDialog.STATUS_WAIT)
                {
                    FileReporterHandler.verbose("W++ "+Thread.currentThread()+" waiting for notify from local nameDlg (GetUsernameDialog)");
                    nameDlg.wait();
                    FileReporterHandler.verbose("W-- "+Thread.currentThread()+" received notify from local nameDlg (GetUsernameDialog)");
                }*/
                // Now check the username, set correct
                Username = nameDlg.getStatus()==GetUsernameDialog.STATUS_OK?
                            nameDlg.getUsername():UN_DEFAULT;
                Password = nameDlg.getStatus()==GetUsernameDialog.STATUS_OK?
                            nameDlg.getPassword():PW_DEFAULT;


                // FileReporterHandler.verbose("L-- "+Thread.currentThread()+" releasing lock on local nameDlg (GetUsernameDialog)");
            //}
        }
        
        
        // Let's go ahead and write to the config file now (since it didn't exist)
        SaveConfiguration();
        
        
    }
    
    /** Requests that the main frame rebuild itself. This allows addons
     * to attempt to update once again, and rebuilds the list of addons,
     * which may have been altered since program start.
     */
    public synchronized static void rebuild()
    {
        f.SetStatus("Rebuilding process list...");
        f.SetProgress(-1, -1, -1);
        f.clean();
        if (f.getFrameState() != MainFrame.COMPLETE_STATE && f.getFrameState() != MainFrame.PANIC_STATE)
            // We can try to recover from a panic error by rebuilding...
        {
            FileReporterHandler.verbose("L?? "+Thread.currentThread()+" requests lock on Main.f (MainFrame)");
            synchronized(f)
            {
                FileReporterHandler.verbose("L++ "+Thread.currentThread().getName()+" gained lock on Main.f (MainFrame)");
                f.setFrameState(MainFrame.DESTROY_STATE);

                // Now we wait for the frame to report back ready
                try
                {
                    while (f.getFrameState() == MainFrame.DESTROY_STATE)
                    {
                        FileReporterHandler.verbose("W++ "+Thread.currentThread().getName()+" waits for notification on Main.f (MainFrame)");
                        f.wait();
                        FileReporterHandler.verbose("W-- "+Thread.currentThread().getName()+" gained notification on Main.f (MainFrame)");
                    }
                }
                catch (InterruptedException e)
                {
                    FileReporterHandler.verbose("W!! "+Thread.currentThread().getName()+" wait interrupted on Main.f (MainFrame)");
                    FileReporterHandler.err("Unexpected InterruptedException");
                    FileReporterHandler.dump(e);
                }
                FileReporterHandler.verbose("L-- "+Thread.currentThread().getName()+" releases lock on Main.f (MainFrame)");
            }   
        }
        try
        {
            // FIXME - FIXED: This causes a (rather large) recursive call, which results
            // in deadlocks. We can't call in this manner, but we could mark it
            // for running. But we need to let this thread get out of the class
            // or else we're going to have this thread and the new one asking
            // for the same object at the same time.
            //buildMainFrame();
            Thread t = new Thread(new Main());
            t.setDaemon(false);
            FileReporterHandler.verbose("T++ "+Thread.currentThread()+
                    " creates and runs thread "+t.getName());
            t.start();
            
            
        }
        catch (Exception e)
        {
            FileReporterHandler h = new FileReporterHandler();
            h.uncaughtException(Thread.currentThread(), e);
        }
        
    }
    
    /** Gets an ID representing the current time, for use 
     * with filename generation
     *
     * @return the string ID representing a timestamp
     */
public static String filetime()
    {
        Calendar currentTime = java.util.Calendar.getInstance();
        
        String formatTime = String.format("%04d-%02d-%02d_%02d-%02d-%02d-%03d",
                currentTime.get(Calendar.YEAR), 
                currentTime.get(Calendar.MONTH),
                currentTime.get(Calendar.DATE),
                currentTime.get(Calendar.HOUR_OF_DAY),
                currentTime.get(Calendar.MINUTE),
                currentTime.get(Calendar.SECOND),
                currentTime.get(Calendar.MILLISECOND)
        );
        return formatTime;
    }
    
    /** Runs this class as a new thread. This is used to make a new thread
     * during a rebuild() so we can start our process over
     */
    public void run()
    {
        try
        {
            buildMainFrame();
        }
        catch (Exception E)
        {
            Thread.currentThread().getUncaughtExceptionHandler().uncaughtException(Thread.currentThread(), E);
        }
    }

    /**
     * Writes the configuration data to the configuration file,
     * creating it if necessary
     * @throws <tt>IOException</tt> if an I/O error occurred
     */
    public static void SaveConfiguration() 
        throws IOException
    {
        java.io.File configFile = new File(CFG_FILE);
        
        if (configFile.exists() && !configFile.delete())
            throw new IOException("Cannot recreate configuration file - code 1");
        if (!configFile.createNewFile())
            throw new IOException("Cannot recreate configuration file - code 2");
        
        BufferedWriter fw = new BufferedWriter(new FileWriter(configFile));
        // Write down this version
        fw.write(CONFIG_VERSION);
        fw.newLine();
        // Now write out wow path
        fw.write(wowPath.getAbsolutePath());
        fw.newLine();
        // And write the username to the file
        if (Username.compareTo(UN_DEFAULT)!=0)
        {
            fw.write(Username);
            fw.newLine();
            fw.write(Password);
            fw.newLine();
        }
        fw.flush();
        fw.close();
    }

    
    
}
