/*
 * MainFrame.java
 *
 * Implements the main frame which the user will see. Includes the
 * addon update progress, list of addons, menu bar, and other UI features
 * which make the program accessible. This is the frame immediately
 * visible at login.
 *
 * Created on March 1, 2007, 1:27 PM
 *
 * Copyright (c) 2007-2008 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;
import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.Vector;

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * Implements the main frame which the user will see. Includes the
 * addon update progress, list of addons, menu bar, and other UI features
 * which make the program accessible. This is the frame immediately
 * visible at login.
 *
 * @author m.delbuono
 */
public class MainFrame extends javax.swing.JFrame {
    
    private static final int SCROLL_PIXELS = 20;    // num pixels to mouse wheel scroll
    
    /** Creates new form MainFrame */
    public MainFrame() {
        threads = new ThreadGroup("UpdateProcess"); 
        threads.setDaemon(false);
        initComponents();
        //this.setIconImage(Main.icon.getImage());
    }
    
    /*
    public void SetPath(File interfaceDir)
    {
        path = interfaceDir;
    }
    
    protected File path;
     **/
    //protected ScrollListPane list;
    
    protected Vector<AddonUpdateProcess> addons = new Vector<AddonUpdateProcess>();
    
    /** The threads which are running to handle the update process */
    protected ThreadGroup threads;   
    /** The threads which are currently running */
    protected volatile Vector<Runnable> runnablesPending = new Vector<Runnable>();
    /** The threads which are pending start */
    protected volatile Vector<Thread> runnablesRunning = new Vector<Thread>();
    //protected volatile Vector<AddonUpdateProcess> processesRunning = new Vector<AddonUpdateProcess>();
    private static final int MAX_THREADS = 5;
    private int running = 0;
    
    private static final String AUP_STRING = "AUP-";
    
    /** The dependents which have been noticed by various processes
     * and should be ignored by future processes */
    public volatile Vector<String> dependents = new Vector<String>();
    /** Status codes */
    public static final int    
                        INACTIVE_STATE = 0,
                        NORMAL_STATE = 1,
                        INTERRUPTED_STATE = 2,
                        COMPLETE_STATE = 3,
                        SHUTDOWN_STATE = 4,
                        DESTROY_STATE = 5,
                        PANIC_STATE = -1;
    
    /** The current state of the frame */
    private volatile int state = INACTIVE_STATE;
    
    /** Sets the current state of the frame
     * @param state the frame state
     */
    public synchronized void setFrameState(int state)
    {
        String statename = null;
        switch(state)
        {
            case INACTIVE_STATE:
                statename = "INACTIVE";
                break;
            case NORMAL_STATE:
                statename = "NORMAL";
                break;
            case INTERRUPTED_STATE:
                statename = "INTERRUPTED";
                break;
            case COMPLETE_STATE:
                statename = "COMPLETE";
                break;
            case SHUTDOWN_STATE:
                statename = "SHUTDOWN";
                break;
            case DESTROY_STATE:
                statename = "DESTROY";
                break;
            case PANIC_STATE:
                statename = "PANIC";
                break;
                
        }
        FileReporterHandler.verbose("S<< "+Thread.currentThread().getName()+" sets main frame state to state "+
                (statename==null?state:statename));
        this.state = state;
    }
    
    /** Returns the current state code of this frame
     * @return the current state of the frame
     */
    public int getFrameState()
    {
        return state;
    }
    
        
    /** Gets the component pane, which is the main window in which 
     * all <tt>AddonUpdateProcess</tt>es can be seen.
     * @return the component pane
     */
    public JPanel getComponentPane()
    {
        return (JPanel)(this.progressListScrollPane.getViewport().getView());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        progressListScrollPane = new javax.swing.JScrollPane();
        statusField = new javax.swing.JTextField();
        statusProgress = new javax.swing.JProgressBar();
        jButton1 = new javax.swing.JButton();
        btnUpdate = new javax.swing.JButton();
        btnInstall = new javax.swing.JButton();
        btnBackup = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        menuFile = new javax.swing.JMenu();
        fileRegister = new javax.swing.JMenuItem();
        fileUpdate = new javax.swing.JMenuItem();
        fileInstall = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        jMenuItem1 = new javax.swing.JMenuItem();
        menuBackup = new javax.swing.JMenu();
        backupManual = new javax.swing.JMenuItem();
        backupRevert = new javax.swing.JMenuItem();
        menuHelp = new javax.swing.JMenu();
        helpAbout = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setIconImage(Main.icon);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowOpened(java.awt.event.WindowEvent evt) {
                formWindowOpened(evt);
            }
        });

        JPanel list = new JPanel();
        list.setBackground(Color.WHITE);
        this.progressListScrollPane.getViewport().setView(list);
        list.setLayout(new GridBagLayout());
        progressListScrollPane.addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                progressListScrollPaneMouseWheelMoved(evt);
            }
        });

        statusField.setEditable(false);
        statusField.setText("Loading...");

        jButton1.setText("Done");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        btnUpdate.setText("Update");
        btnUpdate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnUpdateActionPerformed(evt);
            }
        });

        btnInstall.setText("Install New");
        btnInstall.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnInstallActionPerformed(evt);
            }
        });

        btnBackup.setText("Backup");
        btnBackup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnBackupActionPerformed(evt);
            }
        });

        jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/wowiupdater/wowilogo.png")));

        menuFile.setText("File");
        fileRegister.setText("Register Addons...");
        fileRegister.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileRegisterActionPerformed(evt);
            }
        });

        menuFile.add(fileRegister);

        fileUpdate.setText("Update");
        fileUpdate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileUpdateActionPerformed(evt);
            }
        });

        menuFile.add(fileUpdate);

        fileInstall.setText("Install New...");
        fileInstall.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileInstallActionPerformed(evt);
            }
        });

        menuFile.add(fileInstall);

        menuFile.add(jSeparator1);

        jMenuItem1.setText("Quit");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });

        menuFile.add(jMenuItem1);

        jMenuBar1.add(menuFile);

        menuBackup.setText("Backup");
        menuBackup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuBackupActionPerformed(evt);
            }
        });

        backupManual.setText("Create Full Backup...");
        backupManual.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backupManualActionPerformed(evt);
            }
        });

        menuBackup.add(backupManual);

        backupRevert.setText("Revert...");
        backupRevert.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backupRevertActionPerformed(evt);
            }
        });

        menuBackup.add(backupRevert);

        jMenuBar1.add(menuBackup);

        menuHelp.setText("Help");
        helpAbout.setText("About");
        helpAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpAboutActionPerformed(evt);
            }
        });

        menuHelp.add(helpAbout);

        jMenuBar1.add(menuHelp);

        setJMenuBar(jMenuBar1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(progressListScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 679, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(statusField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 597, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jButton1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 76, Short.MAX_VALUE))
                    .add(layout.createSequentialGroup()
                        .add(jLabel1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 333, Short.MAX_VALUE)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, statusProgress, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                .add(btnUpdate)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(btnInstall)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(btnBackup)))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(statusProgress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(btnUpdate)
                            .add(btnInstall)
                            .add(btnBackup)))
                    .add(jLabel1))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(progressListScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 189, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jButton1)
                    .add(statusField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnBackupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnBackupActionPerformed
        this.backupManualActionPerformed(evt);
    }//GEN-LAST:event_btnBackupActionPerformed

    private void btnInstallActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnInstallActionPerformed
        this.fileInstallActionPerformed(evt);
    }//GEN-LAST:event_btnInstallActionPerformed

    private void btnUpdateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnUpdateActionPerformed
        this.fileUpdateActionPerformed(evt);
    }//GEN-LAST:event_btnUpdateActionPerformed

    private void backupManualActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backupManualActionPerformed
        ManualBackupDialog dlg = new ManualBackupDialog(this, true);
        dlg.setLocationRelativeTo(null);
        dlg.setVisible(true);
    }//GEN-LAST:event_backupManualActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        jMenuItem1ActionPerformed(evt); // Forces quit
        Runtime.getRuntime().exit(0);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void backupRevertActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backupRevertActionPerformed
        try
        {
            ReversionDialog dlg = new ReversionDialog(this, true);
            dlg.setLocationRelativeTo(null);
            dlg.setVisible(true);
        }
        catch (Exception E)
        {
            Thread.currentThread().getDefaultUncaughtExceptionHandler().uncaughtException(
                    Thread.currentThread(), E
            );
            return;
        }
        
    }//GEN-LAST:event_backupRevertActionPerformed

    private void menuBackupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuBackupActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_menuBackupActionPerformed

    private void fileInstallActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileInstallActionPerformed
        // User wants to install a new addon. Open up a modal box and ask for it
        // Synchronize so we don't conflict while switching all these addons
        synchronized(this)
        {
            AddonInstallerDialog dlg = new AddonInstallerDialog(this, true);
            // Parse favorites list for anything missing
            Vector<Addon> missing = new Vector<Addon>();
            for (int i = 0; i < Main.favorites.size(); i++)
            {
                try
                {
                    if (!Main.favorites.get(i).isFound())
                        missing.add(Main.favorites.get(i).toAddon());
                }
                catch (IOException e)
                {
                    FileReporterHandler.err("Unexpected IOException checking for missing favorites. Ignoring.");
                    FileReporterHandler.dump(e);
                }
            }
            // Show the dialog with the newly built list
            dlg.set(missing);
            dlg.setLocationRelativeTo(null);        // center
            dlg.setVisible(true);                   // (Modal)

            // Okay so now let's see what they want us to do
            Vector<Addon> list = dlg.get();

            // To do this, we'll go ahead and add them to the queue
            for (int i = 0; i < list.size(); i++)
            {
                Main.installList.add(list.get(i));
            }

            // Now rebuild to install
            Main.rebuild();
        }
        
        
        
    }//GEN-LAST:event_fileInstallActionPerformed

    private void progressListScrollPaneMouseWheelMoved(java.awt.event.MouseWheelEvent evt) {//GEN-FIRST:event_progressListScrollPaneMouseWheelMoved
        int scrollRatio = SCROLL_PIXELS * evt.getUnitsToScroll();
        int x = progressListScrollPane.getViewport().getViewPosition().x;
        int y = progressListScrollPane.getViewport().getViewPosition().y+scrollRatio;
        
        // Now check if that would go too far before doing this
        if (y < 0) 
            y = 0;
        else if (y > progressListScrollPane.getViewport().getViewSize().height -
                progressListScrollPane.getSize().height + 2)
            y = progressListScrollPane.getViewport().getViewSize().height - 
                    progressListScrollPane.getSize().height + 2;
        
        
        progressListScrollPane.getViewport().setViewPosition(new Point(x,y));
        
        
    }//GEN-LAST:event_progressListScrollPaneMouseWheelMoved

    private void fileUpdateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileUpdateActionPerformed
        synchronized(this)
        {
            Main.rebuild();
        }
    }//GEN-LAST:event_fileUpdateActionPerformed

    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
        FileReporterHandler.verbose("L?? "+Thread.currentThread().getName()+" requests lock on 'this'");
        synchronized (this)
        {
            FileReporterHandler.verbose("L++ "+Thread.currentThread().getName()+" obtains lock on 'this'");
            // Interrupt all updates and then shut down
            setFrameState(SHUTDOWN_STATE);
            runnablesPending.clear();
            for (int i = 0; i < runnablesRunning.size(); i++)
            {
                // Send a terminate request to each
                FileReporterHandler.verbose("W## "+Thread.currentThread().getName()+" sends interrupt to runnablesRunning["+i+"]");
                runnablesRunning.get(i).interrupt();            
            }

            // Now we need to wait for them to shut down. Maximum
            // wait time is 10 seconds, then we will force-quit the JVM
            // after logging the problem

            int secondsRemaining = 10;
        
            while (secondsRemaining > 0 && activity())
            {
                try
                {
                    //FileReporterHandler.verbose("W++ "+Thread.currentThread().getName()+" waits for notification from 'this', 1000ms timeout");
                    this.wait(1000);
                    //FileReporterHandler.verbose("W-- "+Thread.currentThread().getName()+" got notification, or timeout");
                }
                catch(Exception e)
                {
                    FileReporterHandler.err("Error during thread lock at quit. Ignoring for 10 seconds.");
                    FileReporterHandler.dump(e);
                }
                secondsRemaining--;
            }
            
            if (secondsRemaining == 0)
            {
                FileReporterHandler.err("Forcibly terminating JVM");
                FileReporterHandler.err("     > User requested quit");
                FileReporterHandler.err("     > 10s timeout elapsed, threads not terminating");
            }
            
        }
        FileReporterHandler.verbose("L-- "+Thread.currentThread().getName()+" releases lock on 'this'");
        this.dispose(); // Disposal of this frame should shut down the program
        
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    /** Determines if there is current activity (if threads are running)
     * @return <tt>true</tt> if activity exists, <tt>false</tt> if not
     */
    private boolean activity()
    {
        boolean retVal = false;
        for (int i = 0; i < runnablesRunning.size(); i++)
            if (runnablesRunning.get(i).isAlive())
                retVal = true;
        
        return retVal;
    }

    private void helpAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpAboutActionPerformed
        // Show a message regarding FSF/GPL code
        JOptionPane.showMessageDialog(this, Main.APPTITLE + " version "+Main.VERSION+
                ", Copyright (C) 2007 Matthew P. Del Buono\n"+
                "Written for WoWInterface at http://www.wowinterface.com/\n\n"+
                "WoWIUpdater comes with ABSOLUTELY NO WARRANTY\nand is "+
                "distributed under the GNU General Public License v2.\n\n"+
                "This is free software, and you are welcome to redistribute\n"+
                "it under certain conditions. See the license included with "+
                "\nthe distribution for details.\n\nIf you did not receive a "+
                "copy of the GNU GPL along with\nthis distribution, please "+
                "write to\n\n     The Free Software Foundation, Inc.,\n     "+
                "51 Franklin Street, Fifth Floor,\n     "+
                "Boston, MA  02110-1301,\n     "+
                "USA\n\nTo contact the author, please send an e-mail to\n"+
                "skullshot@guardianhq.com or contact \"Shirik\" "+
                "via forum\nprivate message at http://www.wowinterface.com",
                
                "About "+Main.APPTITLE+" v"+Main.VERSION,
                JOptionPane.INFORMATION_MESSAGE
         );
        
    }//GEN-LAST:event_helpAboutActionPerformed

    private void fileRegisterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileRegisterActionPerformed
        // User wants to register new addons (those that don't have a WIC)
        // On creation we did the unregister scan, so let's show the frame
        Main.addonListFrame.setVisible(true);
        Main.addonListFrame.setLocationRelativeTo(null);
    }//GEN-LAST:event_fileRegisterActionPerformed

    private void formWindowOpened(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowOpened
        /*list = new ScrollListPane(this.progressListScrollPane);
        this.progressListScrollPane.getViewport().setView(list);        */       
    }//GEN-LAST:event_formWindowOpened
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {   
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }
    
    /** Adjusts the progress bar
     * @param value The value to set the progress bar, or -1 for indeterminate
     * @param min The minimum value representable by the progress bar, or -1 to leave unchanged
     * @param max The maximum value representable by the progress bar, or -1 to leave unchanged
     */
    public void SetProgress(int value, int min, int max)
    {
        if (this.statusProgress == null)
            throw new NullPointerException("progress bar not initialized");
                    
        if (value == -1)
            this.statusProgress.setIndeterminate(true);
        else
        {
            this.statusProgress.setIndeterminate(false);
            this.statusProgress.setValue(value);
        }
        
        if (min != -1)
            this.statusProgress.setMinimum(min);
        
        if (max != -1)
            this.statusProgress.setMaximum(max);
    }
    
    /** Sets the status of the frame
     * @param s the status to display
     */
    public void SetStatus(String s)
    {
        this.statusField.setText(s);
    }
    
    /** Adds an addon to the component pane
     * <br /><b>Note:</b> DOES NOT revalidate the frame. YOU MUST call 
     * validate() after using this function. This is for efficiency when 
     * adding several addons at one time.
     */
    public void AddAddon(Addon a) {
        //list.AddAddon(new AddonUpdateProcess(a));
        AddAddon(new AddonUpdateProcess(a));
    }

    /** Adds an addon process to the component pane.<br />
     * <b>Note:</b> DOES NOT revalidate the frame. YOU MUST call validate() after
     * using this function. This is for efficiency when adding several
     * addons at one time.
     */
    public void AddAddon(AddonUpdateProcess a) {

        GridBagConstraints addonConstraint = new GridBagConstraints();
        GridBagConstraints statusConstraint = new GridBagConstraints();
        GridBagConstraints progressConstraint = new GridBagConstraints();
        
        // Set grid locations
        addonConstraint.gridx = 0; addonConstraint.gridy = addons.size();
        statusConstraint.gridx = 1; statusConstraint.gridy = addons.size();
        progressConstraint.gridx = 2; progressConstraint.gridy = addons.size();
        
        // Force filling full size horizontanlly
        addonConstraint.fill = GridBagConstraints.HORIZONTAL;
        statusConstraint.fill = GridBagConstraints.HORIZONTAL;
        progressConstraint.gridx = GridBagConstraints.HORIZONTAL;
        
        // Force 3 pixel inset
        addonConstraint.insets = new Insets(3,3,3,3);
        statusConstraint.insets = new Insets(3,3,3,3);
        progressConstraint.insets = new Insets(3,3,3,3);
        
        // Force top-left for status fields, top-right for progress bar
        addonConstraint.anchor = GridBagConstraints.LINE_START;
        statusConstraint.anchor = GridBagConstraints.LINE_START;
        progressConstraint.anchor = GridBagConstraints.LINE_END;
        
        // Weighting: 40% for addon, 30% for status, 30% for progress bar
        addonConstraint.weightx = 0.4;
        statusConstraint.weightx = 0.3;
        progressConstraint.weightx = 0.3;
        
        // Add the addon using the constraints specified
        JProgressBar bar = new JProgressBar();
        a.SetProgressBar(bar);
        JLabel status = new JLabel();
        a.SetStatusText(status);
        
        JComponent[] row = new JComponent[3];
        
        
        JPanel panel = (JPanel)this.progressListScrollPane.getViewport().getView();
        
        row[0] = new JLabel(a.GetName());
        panel.add(row[0], addonConstraint);
        row[1] = status;
        panel.add(status, statusConstraint);
        row[2] = bar;
        panel.add(bar, progressConstraint);
        
        a.SetBackground(row);
        
        a.SetProgressConstraint(progressConstraint);
        //this.validate();
        
        // Add to the addons list
        addons.add(a);
        
        
        // And create the thread that will start handling this.
        // Thread is not automatically run; you must start the process
        // via ExecuteUpdate(), which starts the queue.
//        a.SetThread(new Thread(threads, a, a.GetName()+"_Updater"));
        this.QueueThread(a);
        
    }
    
    
    /** Adds a runnable to the queue to process, which is run at ExecuteUpdate().*/
    private void QueueThread(Runnable r)
    {
        runnablesPending.add(r);
    }
    
    /** Executes the thread queue, to start the update process */
    public void ExecuteUpdate()
        throws IOException, java.security.NoSuchAlgorithmException
    {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" entering MainFrame.ExecuteUpdate()");
        setFrameState(NORMAL_STATE);
        SetStatus("Building favorites list");
        statusProgress.setIndeterminate(true);
        // First let's build the favorites list...
        while(true)
        {
            try
            {
                BuildFavoritesList();
                break;
            }

            catch (PatcherConnectException e)
            {
                // Retry username/password
                GetUsernameDialog dlg = new GetUsernameDialog(this, true, 
                      e.getMessage(),               // Error message
                      Main.Username,                // Initial username
                      Main.Password                 // Initial password
                );
                dlg.setLocationRelativeTo(null);    // Center it   
                dlg.setVisible(true);               // modal, get the username/pw
                
                if (dlg.getStatus() == GetUsernameDialog.STATUS_CANCEL)
                {
                    // Cancel by shutting down app
                    FileReporterHandler.out("--- Terminating Application: User failed to provide a valid username and password");
                    System.exit(14);
                }
                
                Main.Username = dlg.getUsername();
                Main.Password = dlg.getPassword();
                Main.SaveConfiguration();
                continue;
            }
            catch (UnsupportedOperationException e)
            {
                // Operation was cancelled, likely failure to connect to server
                setFrameState(MainFrame.COMPLETE_STATE);
                // Notify any waiting objects that we have exitted
                FileReporterHandler.verbose("L++ "+Thread.currentThread().getName()+" requests lock on 'this'");
                synchronized(this)
                {
                    FileReporterHandler.verbose("W## "+Thread.currentThread().getName()+" sends notify to 'this'");
                    this.notifyAll();
                }
                FileReporterHandler.verbose("L-- "+Thread.currentThread().getName()+" releases lock on 'this'");
                return;
            }
        }
        
        int total = runnablesPending.size();
        int lastSize = 0;
        int complete = 0;
        
        SetStatus(total + " updates to run, using 0/"+MAX_THREADS+" threads.");
        statusProgress.setMinimum(0);
        statusProgress.setMaximum(total);
        statusProgress.setValue(0);
        statusProgress.setIndeterminate(false);
        
        // Check to see if we have threads to execute
        while (runnablesPending.size() > 0 && DESTROY_STATE != getFrameState())
        {
            
            
            // Do we have thread space to execute this?
            if (MAX_THREADS > runnablesRunning.size()-Main.suspendedCount)
            {
                String tid = ((AddonUpdateProcess)runnablesPending.get(0)).GetName();
                // Yes we do. Make the thread, run it.
                Thread t = new Thread(  threads,                    // Thread group
                                        runnablesPending.get(0),    // Process
                                        AUP_STRING+tid              // Generate UID
                                     );
                t.setDaemon(true);
//                processesRunning.add((AddonUpdateProcess)runnablesPending.get(0));
                runnablesRunning.add(t);
                runnablesPending.remove(0);
                FileReporterHandler.verbose("T++ "+Thread.currentThread()+" created thread "+t.getName()+"; starting new thread");
                t.start();
                
            }
            
            // Now let's run through the current running threads to
            // see if we can free any
            for (int i = 0; i < runnablesRunning.size(); i++)
            {
                /* BUGFIX: This is happening way too fast, killing
                 * threads immediately on creation
                 *
                AddonUpdateProcess thisProcess = 
                        (AddonUpdateProcess)runnablesRunning.get(i);
                if (!(thisProcess.isRunning()))
                {
                    // This process isn't running. Let's clean it out.
                    runnablesRunning.remove(i);                    
                }
                 
                */
                if (!runnablesRunning.get(i).isAlive())
                {
                    FileReporterHandler.verbose("T-- "+Thread.currentThread().getName()+" detected dead thread "+runnablesRunning.get(i).getName()+"; removing.");
                    runnablesRunning.remove(i);
//                    processesRunning.remove(i);
                    // On each death, let's validate the window and repaint
                    complete++;
                    this.validate();
                    this.repaint();
                }
                
                
            }
            
            /*
            // Grant the lease to the favorites list if it's available
            if (Main.favoritesLock == null)
            {
                for (int i = 0; i < processesRunning.size(); i++)
                {
                    if (processesRunning.get(i).leaseRequested)
                    {
                        Main.favoritesLock = processesRunning.get(i);
                        processesRunning.get(i).leaseRequested = false;
                        break;
                    }
                }
            }
             */
             
             
            // Process frame update so we give some time back
            if (lastSize != runnablesPending.size())    // Optimization
            {
                lastSize = runnablesPending.size();
                SetStatus(complete + "/" + 
                    total + " updates complete, using "+
                    (runnablesRunning.size()-Main.suspendedCount)+"/"+MAX_THREADS+" threads.");
                
            }
            
            statusProgress.setValue(complete);
            this.repaint();
            try
            {
                //FileReporterHandler.verbose("W++ "+Thread.currentThread().getName()+" waits for notification from 'this', "+Main.THREADLOCK+"ms timeout");
                synchronized(this)
                {
                    // Give a little suspension to break the queue
                    // and give back some CPU time. Just a few ms
                    // It actually makes a nicer "flow" too :)
                    this.wait(Main.THREADLOCK); 
                }
                                
                //FileReporterHandler.verbose("W-- "+Thread.currentThread().getName()+" got notify from 'this', or timeout");
            }
            catch(InterruptedException e)
            {
                // In the event that we got an interruption, we'll
                // want to rebuild and re-execute.
                FileReporterHandler.verbose("W!! "+Thread.currentThread().getName()+" caught interruption");
                Rebuild();
            }
            
            if (getFrameState() == INTERRUPTED_STATE)
            {
                Rebuild();
                //return;
            }
        }
        
        // Though we may not have any more threads to queue,
        // we want to wait here until we have no threads left running
        // because we don't want this function to be called again
        // until we're complete (synchronized, remember!)
        while(runnablesRunning.size() > 0 && getFrameState() != DESTROY_STATE)
        {
            for (int i = 0; i < runnablesRunning.size(); i++)
            {
                if (!runnablesRunning.get(i).isAlive())
                {
                    FileReporterHandler.verbose("T-- "+Thread.currentThread().getName()+" detected dead thread "+runnablesRunning.get(i).getName()+"; removing.");
                    runnablesRunning.remove(i);
//                    processesRunning.remove(i);
                    statusProgress.setValue(++complete);
                    // On each death, let's validate the window and repaint
                    this.validate();
                    this.repaint();
                }
            }
            
            if (lastSize != runnablesPending.size())    // Optimization
            {
                lastSize = runnablesPending.size();
                SetStatus(complete + "/" + total + 
                    " updates complete, using "+
                    (runnablesRunning.size()-Main.suspendedCount)+"/"+MAX_THREADS+" threads.");
                
            }
            try
            {
                //FileReporterHandler.verbose("W++ "+Thread.currentThread().getName()+" waits for notification from 'this', "+Main.THREADLOCK+"ms timeout");
                synchronized (this)
                {
                    this.wait(Main.THREADLOCK);
                }
                //FileReporterHandler.verbose("W-- "+Thread.currentThread().getName()+" got notify from 'this', or timeout");
            }
            catch (InterruptedException e)
            {
                FileReporterHandler.verbose("W!! "+Thread.currentThread().getName()+" caught interruption");
                Rebuild();
                //return;
            }
            this.repaint();
        }
        
        if (getFrameState() == DESTROY_STATE)
        {
            Rebuild();
            return;
        }
        
        setFrameState(COMPLETE_STATE);
        SetStatus("Ready");
        statusProgress.setValue(0);
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" exiting MainFrame.ExecuteUpdate()");
    }
    
    /** Determines if the queue is complete
     * @return <tt>true</tt> if the queue is complete, <tt>false</tt> if it is not
     */
    public boolean isComplete()
    {
        return getFrameState()==COMPLETE_STATE;
    }
    
    
    
    
    /** <h3>Rebuilding</h3>
     * Now THIS FUNCTION shows how much fun multithreading really is ^^
     * <br /><br />
     * Yeah... that was sarcasm
     * <br /><br />
     * Ok here's how this goes. One of the threads running an AddonUpdateProcess
     * detected that an addon contained directories which are in the queue
     * elsewhere. That process wrote the appropriate WID data and then
     * raised a flag in the status semaphore which indicated to this thread
     * (the main thread) that we need to handle a case where we need
     * to fix the queue (because future updates would logically overwrite
     * that dependency if they are run, and we want to prevent that.
     * <br /><br />
     * So, what we do here is have this called whenever that semaphore
     * flags such a condition. This function will interrupt all threads
     * which meet the dependent conditions and interrupt them, indicating
     * that they need to shut down ASAP without finishing their task.
     * <br /><br />
     * Synchronization, volatile semaphores, mutexes; oh so fun
     */
    private void Rebuild()
    {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" enters MainFrame.Rebuild()");
        if (getFrameState() == INTERRUPTED_STATE)
            {
            FileReporterHandler.out("");
            FileReporterHandler.out(">>>>IRQ-0          : Rebuild request caught");
            FileReporterHandler.out("                   : Semaphore Dump Follows");
            FileReporterHandler.out("        <sphState> : INTERRUPTED_STATE");
            FileReporterHandler.out("      <dependents> : [Vector<String>]");
            for (int i = 0; i < dependents.size(); i++)
            {
                System.out.format("                   : [%2d] : %s\n",
                        i,dependents.get(i));
            }

            //SetStatus("Conflict detected, sending interrupt signal...");
            statusProgress.setIndeterminate(true);
            //runnablesPending.clear();   // Don't have to do anything with these
            for (int i = 0; i < runnablesRunning.size(); i++)
            {
                String name = runnablesRunning.get(i).getName().substring(AUP_STRING.length());
                cleanDependents(name, true, i);
                /*
                if (0==a.GetName().compareToIgnoreCase(dependents.get(j)))
                {
                    runnablesRunning.get(i).interrupt();
                    a.UpdateStatus("Cancelled - Dependency Detected",
                            0,0,1,AddonUpdateProcess.ERROR);
                }
                else
                    FileReporterHandler.out("   > Ignored "+dependents)
                 */
            }
            for (int i = 0; i < runnablesPending.size(); i++)
            {
                AddonUpdateProcess a =(AddonUpdateProcess)runnablesPending.get(i);
                cleanDependents(a.GetPath(), false, i);
                a.UpdateStatus("Cancelled - Dependency Detected",
                            0,0,1,AddonUpdateProcess.ERROR);
            }
            FileReporterHandler.out("<<<<<IRQ-0          : Handler success -> NORMAL_STATE");
            setFrameState(NORMAL_STATE);
            /*
            SetStatus("Conflict detected, waiting for threads to exit...");
            while(runnablesRunning.size() != 0)
            {
                try
                {
                    this.wait(1000);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }

                for (int i = 0; i < runnablesRunning.size(); i++)
                {
                    if (!runnablesRunning.get(i).isAlive())
                    {
                        runnablesRunning.remove(i);
                        i--;
                    }
                }
            }
            // Okay so we got all threads to exit; let's get the hell out of here
            // (Yeah, it looks easy here. Now check what AddonUpdateProcess has to
            // do to handle these interrupts :P)
             **/
        }
        else if (getFrameState() == DESTROY_STATE)     // Need to fully rebuild frame
        {
            synchronized(this)
            {
                FileReporterHandler.out(">>>>IRQ-0          : Rebuild request caught");
                FileReporterHandler.out("                   : Semaphore Dump Follows");
                FileReporterHandler.out("        <sphState> : DESTROY_STATE");
                cleanThreads();

                FileReporterHandler.verbose("N** "+Thread.currentThread().getName()+" broadcasts notify to 'this' (ALL)");
                this.notifyAll();
                this.getComponentPane().removeAll();

                FileReporterHandler.out("<<<<<IRQ-0          : Handler Success -> INACTIVE_STATE");
                setFrameState(INACTIVE_STATE);
            }
            
        }
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" exits MainFrame.Rebuild()");
    }
    
    private synchronized void cleanDependents(String name, boolean running, int id)
    {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" enters MainFrame.cleanDependents()");
        // If the name has a / on the end, let's remove it
        for (int j = 0; j < dependents.size(); j++)
        {
            String cmp = dependents.get(j).toLowerCase();
            if (cmp.charAt(cmp.length()-1) == '/')
                cmp = cmp.substring(0, cmp.length()-2);
            
            if (name.toLowerCase().contains(cmp))
            {
                if (running)
                {
                    FileReporterHandler.verbose("W## "+Thread.currentThread().getName()+" interrupts MainFrame.runnablesRunning["+id+"]");
                    runnablesRunning.get(id).interrupt();
                    FileReporterHandler.out("               >>> Interrupted running dependent "+name);
                }
                else
                {
                    runnablesPending.remove(id);
                    FileReporterHandler.out("                 > Removed dependent "+name);
                }
                //a.UpdateStatus("Cancelled - Dependency Detected",
                //        0,0,1,AddonUpdateProcess.ERROR);
            }
            /*else
                FileReporterHandler.out("NO: "+name+" vs "+cmp);*/
        }
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" exits MainFrame.cleanDependents()");
    }
    
    private void cleanThreads()
    {
        FileReporterHandler.out("    <size-pending> :"+runnablesPending.size());
        FileReporterHandler.out("    <size-running> :"+runnablesRunning.size());
        runnablesPending.clear();
        for (int i = 0; i < runnablesRunning.size(); i++)
        {
            FileReporterHandler.out(
                    "                 >>> Terminating thread \""+
                    runnablesRunning.get(i).getName()+"\""
            );
            FileReporterHandler.verbose("W## "+Thread.currentThread().getName()+" interrupts "+runnablesRunning.get(i).getName());
            runnablesRunning.get(i).interrupt();
        }
        runnablesRunning.clear();
    }
    
    private void BuildFavoritesList()
        throws PatcherConnectException
    {
        File f = new File(Main.wowPath, Main.FAVS_FILE);
        Main.favorites = new FavoritesList();
        try
        {
            Main.favorites.load(f);
            // Now let's try to match our addon to it
            return;
        }
        catch (IOException e)
        {
            // Oh well, we can't load our favorites file. Let's request one
            // Delete this in case it exists and is just corrupt
            if (f.exists()) f.delete();
        }
        //@todo load favorites from xml here
        try
        {
            this.SetStatus("Retrieving interface UID list...");
            
            boolean prevStatus = statusProgress.isIndeterminate();
            statusProgress.setIndeterminate(true);

            URL favoritesUrl = new URL(Main.WOWI_URL+
                    String.format(Main.FAVS_FORMAT, Main.Password, Main.Username)
            );
            
            FavoritesParser handler = new FavoritesParser();
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();
            
            URLConnection connection = favoritesUrl.openConnection();
            InputStream in = (InputStream)connection.getContent();

// DEBUG CODE
            /*
int a;
System.out.println(favoritesUrl.toString());
while ((a = in.read())!=-1)
    System.out.write(a);

System.exit(-1);
             */
            
            parser.parse(in, handler);
            if (handler.hasError())
                throw new PatcherConnectException(handler.getErrorID(), handler.getError());
            
            Main.favorites.addEntry(handler.getList());
            this.SetStatus("Processing interface UID list...");
            statusProgress.setIndeterminate(prevStatus);
            
        }
        catch(PatcherConnectException e)
        {
            FileReporterHandler.err("Received error from server when obtaining favorites list - "+e.getMessage());
            throw e;
        }
        catch(Exception e)
        {
            FileReporterHandler.err("Caught Error while parsing favorites list");
            FileReporterHandler.dump(e);
            
            this.SetStatus("Unable to read favorites list. Check your internet connection and try again.");
            this.statusProgress.setIndeterminate(false);
            this.statusProgress.setValue(0);
            throw new UnsupportedOperationException("Operation Aborted");
        }
    }
    
    /** Removes all components from the container in preparation for rebuild */
    public void clean()
    {
        JPanel list = (JPanel)progressListScrollPane.getViewport().getView();
        list.removeAll();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem backupManual;
    private javax.swing.JMenuItem backupRevert;
    private javax.swing.JButton btnBackup;
    private javax.swing.JButton btnInstall;
    private javax.swing.JButton btnUpdate;
    private javax.swing.JMenuItem fileInstall;
    private javax.swing.JMenuItem fileRegister;
    private javax.swing.JMenuItem fileUpdate;
    private javax.swing.JMenuItem helpAbout;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JMenu menuBackup;
    private javax.swing.JMenu menuFile;
    private javax.swing.JMenu menuHelp;
    private javax.swing.JScrollPane progressListScrollPane;
    private javax.swing.JTextField statusField;
    private javax.swing.JProgressBar statusProgress;
    // End of variables declaration//GEN-END:variables
    
}
