/*
 * ManualBackupDialog.java
 *
 * Created on April 8, 2007, 6:53 AM
 *
 * Implements a dialog to allow the user to perform a backup of the current
 * Interface/Addons and WTF folder, so that they can be restored later.
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.io.File;
import java.io.IOException;
import java.util.Calendar;
import java.util.Vector;

/**
 * Implements a dialog to allow the user to perform a backup of the current
 * Interface/Addons and WTF folder, so that they can be restored later.
 *
 * @author  m.delbuono
 */
public class ManualBackupDialog extends javax.swing.JDialog 
        implements Runnable
{
    private Thread thisThread = null;
    
    private volatile Vector<File> addonFiles = new Vector<File>();
    private volatile int addonDirectories = 0;
    
    private volatile Vector<File> unregAddons = new Vector<File>();
    private volatile int unregDirectories = 0;
    
    private volatile Vector<File> svFiles = new Vector<File>();
    private volatile int svDirectories = 0;
    
    private volatile Vector<File> configFiles = new Vector<File>();
    private volatile Vector<File> residentZips = new Vector<File>();
    
    private volatile long bytes = 0;
    
    private volatile boolean ready = false;
    
    
    /** Status strings for the log textarea */
    private volatile String s1 = "", s2 = "", s3 = "", s4 = "";
    
    
    // Optimization vars (saves exponential creation)
    private File addonsDir;
    private File svDir;
    private WIDFilter widfilter = new WIDFilter(false);
    private NonZipFilter nzfilter = new NonZipFilter();
    
    /** Creates new form ManualBackupDialog */
    public ManualBackupDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        
        addonsDir =  new File(Main.wowPath, Main.REL_ADDONS);
        svDir = new File(Main.wowPath, Main.REL_SV);
        
        
        // Even if not shown right away, we're going to start scanning
        thisThread = new Thread(this, "MBD-"+Calendar.getInstance().getTimeInMillis());
        FileReporterHandler.verbose("T++ "+Thread.currentThread().getName()+
                " creates thread "+thisThread.getName());
        thisThread.setDaemon(true);
        thisThread.start();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jPanel1 = new javax.swing.JPanel();
        cbSV = new javax.swing.JCheckBox();
        cbUnregistered = new javax.swing.JCheckBox();
        cbConfig = new javax.swing.JCheckBox();
        cbZips = new javax.swing.JCheckBox();
        cbVerify = new javax.swing.JCheckBox();
        btnBackup = new javax.swing.JButton();
        btnExit = new javax.swing.JButton();
        fieldStatus = new javax.swing.JTextField();
        progress = new javax.swing.JProgressBar();
        jLabel1 = new javax.swing.JLabel();
        fieldFolders = new javax.swing.JTextField();
        fieldFiles = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        fieldUncompressed = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        taStatus = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Backup Manager");
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Options"));
        cbSV.setSelected(true);
        cbSV.setText("Backup Saved Variables");
        cbSV.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        cbSV.setEnabled(false);
        cbSV.setMargin(new java.awt.Insets(0, 0, 0, 0));
        cbSV.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbSVActionPerformed(evt);
            }
        });

        cbUnregistered.setSelected(true);
        cbUnregistered.setText("Backup Unregistered Addons");
        cbUnregistered.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        cbUnregistered.setEnabled(false);
        cbUnregistered.setMargin(new java.awt.Insets(0, 0, 0, 0));
        cbUnregistered.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbUnregisteredActionPerformed(evt);
            }
        });

        cbConfig.setSelected(true);
        cbConfig.setText("Backup Updater Configurations");
        cbConfig.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        cbConfig.setEnabled(false);
        cbConfig.setMargin(new java.awt.Insets(0, 0, 0, 0));
        cbConfig.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbConfigActionPerformed(evt);
            }
        });

        cbZips.setSelected(true);
        cbZips.setText("Backup Resident ZIP Archives");
        cbZips.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        cbZips.setEnabled(false);
        cbZips.setMargin(new java.awt.Insets(0, 0, 0, 0));
        cbZips.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbZipsActionPerformed(evt);
            }
        });

        cbVerify.setSelected(true);
        cbVerify.setText("Verify Backup After Writing");
        cbVerify.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        cbVerify.setEnabled(false);
        cbVerify.setMargin(new java.awt.Insets(0, 0, 0, 0));
        cbVerify.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbVerifyActionPerformed(evt);
            }
        });

        btnBackup.setText("Backup");
        btnBackup.setEnabled(false);
        btnBackup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnBackupActionPerformed(evt);
            }
        });

        btnExit.setText("Exit");
        btnExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnExitActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(cbSV)
                    .add(cbUnregistered)
                    .add(cbConfig)
                    .add(cbZips)
                    .add(cbVerify)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(btnBackup, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnExit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 78, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(cbSV)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbUnregistered)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbConfig)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbZips)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbVerify)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 7, Short.MAX_VALUE)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnExit)
                    .add(btnBackup))
                .addContainerGap())
        );

        fieldStatus.setEditable(false);
        fieldStatus.setText("Preparing for backup...");

        jLabel1.setText("Folders to backup:");

        fieldFolders.setEditable(false);
        fieldFolders.setText("Scanning...");

        fieldFiles.setEditable(false);
        fieldFiles.setText("Scanning...");

        jLabel2.setText("Files to backup:");

        jLabel3.setText("Uncompressed Size:");

        fieldUncompressed.setEditable(false);
        fieldUncompressed.setText("Scanning...");

        taStatus.setColumns(20);
        taStatus.setEditable(false);
        taStatus.setFont(new java.awt.Font("Courier", 0, 12));
        taStatus.setRows(3);
        jScrollPane1.setViewportView(taStatus);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(fieldStatus, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 405, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(progress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 133, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(14, 14, 14)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jLabel3)
                                    .add(jLabel2)
                                    .add(jLabel1))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 37, Short.MAX_VALUE)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, fieldFiles)
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, fieldFolders, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 137, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, fieldUncompressed, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 194, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                            .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 327, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(progress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(14, 14, 14)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jLabel1)
                                    .add(fieldFolders, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jLabel2)
                                    .add(fieldFiles, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jLabel3)
                                    .add(fieldUncompressed, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 92, Short.MAX_VALUE))
                            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(fieldStatus, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnBackupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnBackupActionPerformed
        // Create a thread to handle the backup
        Thread t = new Thread(this, "MBD-"+Calendar.getInstance().getTimeInMillis());
        FileReporterHandler.verbose("T++ "+Thread.currentThread().getName()+
                " creates thread "+t.getName());
        // Start it
        t.start();
        
    }//GEN-LAST:event_btnBackupActionPerformed

    private void btnExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnExitActionPerformed
        thisThread.interrupt();
        this.dispose();
    }//GEN-LAST:event_btnExitActionPerformed

    private void cbVerifyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbVerifyActionPerformed
        UpdateControls();
    }//GEN-LAST:event_cbVerifyActionPerformed

    private void cbZipsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbZipsActionPerformed
        UpdateControls();
    }//GEN-LAST:event_cbZipsActionPerformed

    private void cbConfigActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbConfigActionPerformed
        UpdateControls();
    }//GEN-LAST:event_cbConfigActionPerformed

    private void cbUnregisteredActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbUnregisteredActionPerformed
        UpdateControls();
    }//GEN-LAST:event_cbUnregisteredActionPerformed

    private void cbSVActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbSVActionPerformed
        UpdateControls();
    }//GEN-LAST:event_cbSVActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new ManualBackupDialog(new javax.swing.JFrame(), true).setVisible(true);
            }
        });
    }
    
    /** Sets the status of the dialog.
     *@param status A string to set the current status to. If this is
     *<tt>null</tt>, the status will not be changed. If not <tt>null</tt>,
     *the status will be updated and the will be appended to the
     *log box.
     *@param start The minimum value for the progress bar, or -1 if it should
     *not be changed.
     *@param stop The maximum value for the progress bar, or -1 if it should
     *not be changed, or -2 if the progress bar should be set to an
     *indeterminate state. Specifying any non-negative value will result in
     *it becoming non-indeterminate.
     *@param val The value to set the progress bar to, or -1 if it should
     *not be changed, or -2 if it should be stepped by 1.
     */
    public void SetStatus(String status, int start, int stop, int val)
    {
        if (status != null)
        {
            // Update status fields
            s1 = s2;
            s2 = s3;
            s3 = s4;
            s4 = status;
            // Build display text
            String display = s1 + "\n" + s2 + "\n" + s3 + "\n" + s4;
            taStatus.setText(display);
            fieldStatus.setText(status);
        }
        
        if (start >= 0)
        {
            progress.setMinimum(start);
        }
        
        if (stop >= 0)
        {
            progress.setIndeterminate(false);
            progress.setMaximum(stop);
        }
        else if (stop == -2)
        {
            progress.setIndeterminate(true);
        }
        
        if (val >= 0)
        {
            progress.setValue(val);
        }
        else if (val == -2)
        {
            progress.setValue(progress.getValue()+1);
        }
            
    }

    /** Starts this Runnable. This Runnable will prepare all information for this dialog,
     * including folders and files to backup, etc.
     */
    public void run() {
        if (!ready)
        {
            SetStatus(null, -1, -2, -1);
            // Start by scanning the addons folder
            processList(addonsDir.listFiles());
            if (Thread.currentThread().isInterrupted())
            {
                FileReporterHandler.verbose("T## "+Thread.currentThread().getName()+
                        " received interrupt. Terminating execution.");
                return;
            }

            // Now the SV folder
            processList(svDir.listFiles());

            if (Thread.currentThread().isInterrupted())
            {
                FileReporterHandler.verbose("T## "+Thread.currentThread().getName()+
                        " received interrupt. Terminating execution.");
                return;
            }

            // Now we've got all of the data, let's write it to the controls
            UpdateControls();
            progress.setIndeterminate(false);
            SetBackupEnabled(true);
            ready = true;       // Next time we run this, we backup!
        }
        else
        {
            SetBackupEnabled(false);
            try
            {
                RunBackup();
            }
            catch (IOException e)
            {
                FileReporterHandler.err("Error writing manual backup");
                FileReporterHandler.dump(e);
            }
            catch (InterruptedException e)
            {
                // Ignore. If we got interrupted, we just have to break out
                FileReporterHandler.out(">> Caught IRQ-4, Terminating Backup");
            }
            SetBackupEnabled(true);
        }
    }
    
    /** Processes a list of files and separates them into the appropriate
     * vectors. This acts recursively, that is, any directories found
     * in the list will be handled as well
     *@param list the list of <tt>File</tt> objects to process
     */
    private void processList(File[] list)
    {
        
        for (int i = 0; i < list.length; i++)
        {
            if (list[i].isDirectory())
            {
                fieldStatus.setText("Scanning folder '"+list[i].getName()+"'");
                this.repaint();
                if (isRegisteredFile(list[i]))
                    addonDirectories++;
                else if (isUnregisteredAddonFile(list[i]))
                    unregDirectories++;
                else if (isSVFile(list[i]))
                    svDirectories++;
                
                processList(list[i].listFiles());
            }
            else
            {
                // This is a file! Figure out what type of file it is
                if (isRegisteredFile(list[i]))
                {
                    if (!nzfilter.accept(list[i]))
                        residentZips.add(list[i]);
                    else if (widfilter.accept(list[i]))
                        configFiles.add(list[i]);
                    else
                        addonFiles.add(list[i]);
                }
                else if (isUnregisteredAddonFile(list[i]))
                {
                    if (!nzfilter.accept(list[i]))
                        residentZips.add(list[i]);
                    else
                        unregAddons.add(list[i]);
                }
                else if (isSVFile(list[i]))
                    svFiles.add(list[i]);
                
                
                bytes += list[i].length();
            }
        }
    }

    /** Determines if the passed in <tt>File</tt> is part of a registered
     * addon.
     * @param file the <tt>File</tt> object to test
     * @return <tt>true</tt> if this file or directory is part of a registered
     * addon, <tt>false</tt> otherwise.
     */
    private boolean isRegisteredFile(File file) {
        // Check to see if this is an addon directory
        while (AddonUpdateProcess.isChild(file,addonsDir))
        {
            if (file.isDirectory())
            {
                File[] thisList = file.listFiles();
                for (int j = 0; j < thisList.length; j++)
                {
                    if (widfilter.accept(thisList[j]))
                        return true;
                }
            }
            file = file.getParentFile();
        }
        return false;
        
    }
    
    /** Determines if the passed in <tt>File</tt> is part of an unregistered
     * addon.
     * @param file the <tt>File</tt> object to test
     * @return <tt>true</tt> if this file or directory is part of an unregistered
     * addon, <tt>false</tt> otherwise.
     */
    private boolean isUnregisteredAddonFile(File file) {
        if (AddonUpdateProcess.isChild(file,addonsDir))
        {
            // Now check to see if it has a WID
            while (!addonsDir.equals(file))
            {
                if (file.isDirectory())
                {
                    File[] thisList = file.listFiles();
                    for (int j = 0; j < thisList.length; j++)
                    {
                        if (widfilter.accept(thisList[j]))
                            return false;
                    }
                }
                file = file.getParentFile();
            }
            return true;
        }
        return false;
        
    }

    /** Determines if the passed in <tt>File</tt> is in the saved variables
     * folder.
     * @param file the <tt>File</tt> object to test
     * @return <tt>true</tt> if this file or directory is in the saved variables
     * folder, <tt>false</tt> otherwise.
     */
    private boolean isSVFile(File file) {
        // Check to see if this is a SV directory
        return (AddonUpdateProcess.isChild(file,
                new File(Main.wowPath, Main.REL_SV)));
    }

    /** Updates the controls that identify how many folders/directories
     * will be backed up, and sets the status bar to "ready"
     */
    private void UpdateControls() {
        int filesCount, dirsCount;
        // first, base off registered addons, which are always backed up
        filesCount = addonFiles.size();
        dirsCount = addonDirectories;
        
        // do we backup saved vars?
        if (cbSV.isSelected())
        {
            filesCount += svFiles.size();
            dirsCount += svDirectories;
        }
        // do we backup unregistered addons?
        if (cbUnregistered.isSelected())
        {
            filesCount += unregAddons.size();
            dirsCount += unregDirectories;
        }
        // do we backup config files?
        if (cbConfig.isSelected())
        {
            filesCount += configFiles.size();
        }
        // do we backup resident zips?
        if (cbZips.isSelected())
        {
            filesCount += residentZips.size();
        }
        
        // Now go update the controls
        fieldFiles.setText(String.valueOf(filesCount));
        fieldFolders.setText(String.valueOf(dirsCount));
        fieldUncompressed.setText(String.valueOf((int)(bytes/1024/1024))+" MB");
        fieldStatus.setText("Ready");
        
    }
    
    /** Enables or disables the config checkboxes and backup button.
     * @param isEnabled <tt>true</tt> if the boxes and buttons are to
     * be enabled, <tt>false</tt> if they are to be disabled.
     */
    private void SetBackupEnabled(boolean isEnabled)
    {
        btnBackup.setEnabled(isEnabled);
        cbConfig.setEnabled(isEnabled);
        cbSV.setEnabled(isEnabled);
        cbUnregistered.setEnabled(isEnabled);
        cbVerify.setEnabled(isEnabled);
        cbZips.setEnabled(isEnabled);
    }

    /** Performs the backup based on options selected 
     * @throws IOException if an I/O error occurred
     * @throws InterruptedException if the thread was interrupted while
     * backing up or while waiting for a backup manager lock
     */
    private void RunBackup() throws IOException, InterruptedException {
        SetStatus("Finalizing list...", 0, -2, 0);
        // Build a main list
        Vector<File> backupList = new Vector<File>();
        //int size = addonFiles.size();
        backupList.addAll(addonFiles);
        if (cbUnregistered.isSelected())
            backupList.addAll(unregAddons);
        if (cbSV.isSelected())
            backupList.addAll(svFiles);
        if (cbZips.isSelected())
            backupList.addAll(residentZips);
        if (cbConfig.isSelected())
            backupList.addAll(configFiles);
        
        // Now we have a list, let's go form the backup
        File backupsDir = new File(Main.wowPath, Main.REL_BACKUPS);
        File zipfile = new File(backupsDir, "backup-manual-"+Main.filetime()+".zip");
        zipfile.createNewFile();
        AddonExtractor zip = new AddonExtractor(zipfile);
        zip.OpenForWriting();
            
        for (int i = 0; i < backupList.size(); i++)
        {
            if (Thread.currentThread().isInterrupted())
                throw new InterruptedException("Interrupted during backup");
            
            SetStatus("Writing file "+backupList.get(i).getName()+"...",
                    0, backupList.size(), -2);
            zip.add(backupList.get(i), 
                    AddonUpdateProcess.getRelativePath(
                        backupList.get(i),
                        Main.wowPath,
                        '/'
                    )
            );
        }
        SetStatus("Finalizing archive...", -1, -2, -1);
        zip.Close();
        Main.backupManager.OpenForAppending();
        Main.backupManager.AddBackup(zipfile, null, "Manual Complete Backup");
        Main.backupManager.Close();
        SetStatus("Backup Complete!", 0, 1, 1);
        
        
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnBackup;
    private javax.swing.JButton btnExit;
    private javax.swing.JCheckBox cbConfig;
    private javax.swing.JCheckBox cbSV;
    private javax.swing.JCheckBox cbUnregistered;
    private javax.swing.JCheckBox cbVerify;
    private javax.swing.JCheckBox cbZips;
    private javax.swing.JTextField fieldFiles;
    private javax.swing.JTextField fieldFolders;
    private javax.swing.JTextField fieldStatus;
    private javax.swing.JTextField fieldUncompressed;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JProgressBar progress;
    private javax.swing.JTextArea taStatus;
    // End of variables declaration//GEN-END:variables
    
}
