/*
 * MatcherSuspender.java
 *
 * Suspends a thread until a user selects a match which was found,
 * or enters an alternate UID if the matching ones were invalid.
 *
 * Created on March 11, 2007, 3:18 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.*;
import java.awt.*;

/**
 * Suspends a thread until a user selects a match which was found,
 * or enters an alternate UID if the matching ones were invalid.
 * @author m.delbuono
 */

public class MatcherSuspender extends ResolverSuspender {
    Vector<FavoritesEntry> matches;

    private Addon addon;
    /** Creates a new instance of MatcherSuspender */
    public MatcherSuspender(Vector<FavoritesEntry> m, Addon a) {
        matches = m;
        addon = a;
    }
    /** Status codes */
    public static final int
            WAIT_STATUS = 0,
            OK_STATUS = 1,
            CANCEL_STATUS = 2;
    
    private MatchesFrame frame;
    
    private int status = WAIT_STATUS;
    
    /** Sets the current status 
     * @param status the status code to set
     */
    public void setStatus(int status)
    {
        this.status = status;
    }
    
    /** Suspends this <tt>Suspender</tt>, waiting for user input
     * @param c the <tt>GridBagConstraints</tt> to control this <tt>Suspender</tt>
     * @param p the <tt>JPanel</tt> which this <tt>Suspender</tt> controls
     * @param b the <tt>JProgressBar</tt> indicating the status of this <tt>Suspender</tt>
     * @throws java.util.concurrent.CancellationException if the suspension was
     * cancelled by the user
     * @throws InterruptedException if the <tt>Suspender</tt> was interrupted
     * while in a wait loop
     */
    public synchronized void Suspend(GridBagConstraints c, JPanel p, JProgressBar b)
        throws java.util.concurrent.CancellationException, InterruptedException
    {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" enters MatcherSuspender.Suspend");
        constraints = c;
        this.p = p;
        oldBar = b;
        status = WAIT_STATUS;
        
        
        GenerateResolutionButton("Resolve");
        
        while (status != OK_STATUS)
        {
            Main.f.repaint();
            try
            {
                FileReporterHandler.verbose("W++ "+Thread.currentThread().getName()+" waits for notification from 'this'");
                this.wait();
                FileReporterHandler.verbose("W-- "+Thread.currentThread().getName()+" got notify from 'this'");
            }
            catch (InterruptedException e)
            {
                FileReporterHandler.verbose("W!! "+Thread.currentThread().getName()+" interrupted in wait loop; cascading throw");
                throw e;
            }
        }
        
        addon.uid = frame.GetUID();
        DisposeResolutionButton();
        
        
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" exits MatcherSuspender.Suspend");
    }

    /** Handles actions performed by the user, through an ActionListener
     * @param e the action that occurred
     */
    public synchronized void actionPerformed(ActionEvent e) {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" enters MatcherSuspender.actionPerformed");
        if (e.getSource() == resolveBtn)
        {
            DisableResolve();
            Resolve();
        }
        else
        {
            EnableResolve();
            FileReporterHandler.verbose("W** "+Thread.currentThread().getName()+" sending notify to 'this'");
            this.notifyAll();
        }
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" exits MatcherSuspender.actionPerformed");
    }

    private synchronized void Resolve() {
        FileReporterHandler.verbose("C++ "+Thread.currentThread().getName()+" enters MatcherSuspender.Resolve");
        
        frame = new MatchesFrame();
        frame.setCallbacks(this);        
        frame.setLocationRelativeTo(null);      // centered
        frame.validate();
        frame.identify(addon, matches);
        frame.setVisible(true);
        
        
        FileReporterHandler.verbose("C-- "+Thread.currentThread().getName()+" exits MatcherSuspender.Resolve");
    }
    
}
