/*
 * NonZipFilter.java
 *
 * Implements a FileFilter which will accept all files and directories
 * except those ending in .zip (i.e., any non-zip files).
 *
 * Created on March 4, 2007, 7:21 PM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;
import java.io.File;
import java.io.FileFilter;
/**
 * Implements a FileFilter which will accept all files and directories
 * except those ending in .zip (i.e., any non-zip files).
 * @author m.delbuono
 */
public class NonZipFilter implements FileFilter {
    
    /** Creates a new instance of NonZipFilter */
    public NonZipFilter() {
    }

    /** Determines if this is a non-zip file
     * @param pathname the <tt>File</tt> to test
     * @return <tt>true</tt> if this is a non-zip file, <tt>false</tt> if it is a zip file
     */
    public boolean accept(File pathname) {
        if (pathname.isDirectory())
            return true;
        
        String ext = pathname.getName().substring(pathname.getName().length()-4);
        
        if (0==ext.compareToIgnoreCase(".zip"))
            return false;
        else
            return true;
    }
    
}
