/*
 * RARExtractor.java
 *
 * Implements a reader for a RAR compressed archive. Utilizes the
 * unrarlib, available at http://www.unrarlib.org, under the terms of the
 * GNU General Public License. Relies on C++ native code.
 *
 * Created on April 1, 2007, 4:44 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 *
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;
/**
 * Implements a reader for a RAR compressed archive. Utilizes the
 * unrarlib, available at http://www.unrarlib.org, under the terms of the
 * GNU General Public License. Relies on C++ native code.
 *
 * @author m.delbuono
 */
public class RARExtractor {
    static
    {
        System.loadLibrary("rarlib");
    }
    
    private File rar = null;
    private String thisFile;
    private int currentElement;
    private String[] list = null;
    private boolean[] directory = null;
    /** Creates a new instance of RARExtractor */
    public RARExtractor(File rar) {
        this.rar = rar;
        String[] l = load(rar);
        // Parse the list. Each one starts with an f or a d. f = file, d = dir
        list = new String[l.length];
        directory = new boolean[l.length];
        for (int i = 0; i < l.length; i++)
        {
            list[i] = l[i].substring(1);
            directory[i] = (l[i].charAt(0)=='d');
        }
        
    }
    
    // Dumps the list of strings (testing purposes)
    public void dump()
    {
        for (int i = 0; i < list.length; i++)
            System.out.println((directory[i]?"Directory":"File") + " " + list[i]);
    }
    
    // Tests the load of the DLL
    native static public void test();
    
    // Gets an entry
    native private static String[] load(String path);
    private static String[] load(File rar)
    {
       /* Vector<String> l = new Vector<String>();
        char[] r = load(rar.getAbsolutePath());
        // Make strings, add to list.
        // Null byte = end of string
        // Array will always be terminated by a null byte
        String thisString = "";
        for (int x = 0; x < r.length; x++)
        {
            if (r[x] == 0)
            {
                l.add(thisString);
                thisString = "";
            }
            else
                thisString += r[x];
        }
        
        return l;
        **/
        return load(rar.getAbsolutePath());
    }
    
    // Extracts an entry into an array of bytes
    native public static int[] get(String file, String rar);
    public int[] get(int index)
    {
        System.out.println(list[index] + " "+rar.getAbsolutePath());
        return get(list[index], rar.getAbsolutePath());
    }
    public int[] get()
    {
        return get(thisFile, rar.getAbsolutePath());
    }
    
    
    // Gets the first entry and initializes pointer
    public String getFirstEntry()
    {
        thisFile = getEntry(currentElement = 0);
        return thisFile;
    }
            
    // Gets the next entry
    public String getNextEntry()
    {
        thisFile = getEntry(++currentElement);
        return thisFile;
    }
    
    public String getEntry(int i)
    {
        return list[currentElement];
    }
}


