/*
 * ReversionDialog.java
 *
 * Implements a dialog to show versions of backups which the user may
 * revert to, and allows the user to revert to those backups if desired.
 *
 * Created on April 4, 2007, 8:33 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

import java.io.IOException;
import java.util.Vector;
import javax.swing.JList;
import javax.swing.ListSelectionModel;

/**
 * Implements a dialog to show versions of backups which the user may
 * revert to, and allows the user to revert to those backups if desired.
 * @author  m.delbuono
 */
public class ReversionDialog extends javax.swing.JDialog {
    private Vector<Integer> uids = null;
    private Vector<BackupEntry>[] entries = null;
    private Vector<BackupEntry> fullList = null;
    //Vector<String> comboBoxList = null;
    private JList list;
    
    /** Creates a <tt>ReversionDialog</tt>
     * @param parent the parent of this dialog
     * @param modal <tt>true</tt> if this is a modal dialog, <tt>false</tt> if
     * not
     * @SuppressWarnings unchecked
     */
    @SuppressWarnings("unchecked")
    public ReversionDialog(java.awt.Frame parent, boolean modal) 
        throws IOException, InterruptedException
    {
        super(parent, modal);
        initComponents();
        
        // Load data from the backup manager into the dlg
        Main.backupManager.OpenForReading();
        Main.backupManager.load();
        
        // Initialize UID vector
        uids = new Vector<Integer>();
        fullList = new Vector<BackupEntry>();
        //comboBoxList = new Vector<String>();
        //comboBoxList.add("All Backups");
        addonBox.addItem(new CBData("All Backups", -1));
        
        // First, get a vector of UIDs
        // (at same time, build the full list in reverse order)
        BackupEntry thisEntry;
        int c = 0;
        while ((thisEntry = Main.backupManager.next())!=null)
        {
            boolean duplicate = false;
            int thisUID = thisEntry.uid;
            fullList.add(0, thisEntry);
            // Scan to see if it's in the vector already
            for (int i = 0; i < uids.size(); i++)
            {
                if (thisUID == uids.get(i))
                    duplicate = true;   // Skip this one, we already have it
            }
            if (duplicate)
                continue;
            
            // It's not, so add it
            uids.add(thisUID);
            //comboBoxList.add(thisUID + " - " + thisEntry.name);
            addonBox.addItem(new CBData(thisUID==0?thisEntry.name:(thisUID + " - " + thisEntry.name), c++));
        }
        
        // Okay we have a list of UIDs. Make the array
        entries = new Vector[uids.size()];
        // Init each one
        for (int i = 0; i < entries.length; i++)
            entries[i] = new Vector<BackupEntry>();
        
        // Now load each addon into the appropriate vector
        // We add in reverse order to make chronological
        Main.backupManager.reset();
        while ((thisEntry = Main.backupManager.next())!=null)
        {
            int thisUID = thisEntry.uid;
            // Find this UID in our UID vector
            for (int i = 0; i < uids.size(); i++)
            {
                if (uids.get(i)==thisUID)
                {
                    entries[i].add(0, thisEntry);
                    break;  // No sense continuing, it's just a waste of time
                }
            }
        }
        
        // Okay we've got our vectors. Now initialize controls.
        // We'll start with the 'all backups' list selected
        addonBox.setSelectedIndex(0);
        list.clearSelection();
        list.setListData(fullList);
        
        Main.backupManager.Close();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        addonBox = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        btnExit = new javax.swing.JButton();
        btnRestore = new javax.swing.JButton();
        jTextField1 = new javax.swing.JTextField();
        jProgressBar1 = new javax.swing.JProgressBar();
        restoreSavedVars = new javax.swing.JCheckBox();
        jScrollPane1 = new javax.swing.JScrollPane();
        jLabel3 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Restore Backup...");
        addonBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addonBoxActionPerformed(evt);
            }
        });

        jLabel1.setText("Select a backup to restore...");

        jLabel2.setText("Addon");

        btnExit.setText("Exit");
        btnExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnExitActionPerformed(evt);
            }
        });

        btnRestore.setText("Restore");
        btnRestore.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRestoreActionPerformed(evt);
            }
        });

        jTextField1.setEditable(false);
        jTextField1.setText("Ready");

        restoreSavedVars.setText("Restore Saved Variables With Addon");
        restoreSavedVars.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        restoreSavedVars.setMargin(new java.awt.Insets(0, 0, 0, 0));

        list = new JList();
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jScrollPane1.setViewportView(list);

        jLabel3.setText("Note: Choosing an event will restore the selected addon to its state before that event occurred.");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 465, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(addonBox, 0, 430, Short.MAX_VALUE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 217, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jProgressBar1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 14, Short.MAX_VALUE)
                        .add(btnRestore)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnExit))
                    .add(layout.createSequentialGroup()
                        .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 221, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 51, Short.MAX_VALUE)
                        .add(restoreSavedVars))
                    .add(jLabel3))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(restoreSavedVars))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(addonBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 130, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(btnExit)
                        .add(btnRestore)
                        .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jProgressBar1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel3)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnExitActionPerformed
        this.dispose();
    }//GEN-LAST:event_btnExitActionPerformed

    private void btnRestoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRestoreActionPerformed
        // Make a reversion thread and do it
        BackupEntry be = (BackupEntry) list.getSelectedValue();          
        AddonReverter r = new AddonReverter(
                be, 
                restoreSavedVars.isSelected(),
                jTextField1,
                jProgressBar1,
                btnExit
        );
        Thread t = new Thread(r, "BRP-"+be.timestamp);
        // Now start the process
        FileReporterHandler.verbose("T++ "+Thread.currentThread().getName()+
                " creates thread "+t.getName());
        t.start();
    }//GEN-LAST:event_btnRestoreActionPerformed

    private void addonBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addonBoxActionPerformed
        // Combo box selection has changed, so let's display the correct list
        list.clearSelection();
        int i = addonBox.getSelectedIndex();
        // If i == 0, we should use the full list
        // Otherwise we should use entries[i-1] (-1 because we start at
        // 1 instead of 0, since 0 is taken up by the full list)
        if (i == 0)
            list.setListData(fullList);
        else
            list.setListData(entries[i-1]);
    }//GEN-LAST:event_addonBoxActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                try
                {
                    new ReversionDialog(new javax.swing.JFrame(), true).setVisible(true);
                }
                catch (Exception e)
                {
                    Thread.getDefaultUncaughtExceptionHandler().uncaughtException(Thread.currentThread(), e);
                    return;
                }
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox addonBox;
    private javax.swing.JButton btnExit;
    private javax.swing.JButton btnRestore;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JCheckBox restoreSavedVars;
    // End of variables declaration//GEN-END:variables
    
}

// Encapsulator for the data in combo boxes
class CBData
{
    private String val;
    private int id;
    public CBData(String s, int id)
    {
        val = s;
        this.id = id;
    }
    
    public String toString()
    {
        return val;
    }
    
    public int GetID()
    {
        return id;
    }
}
