/*
 * TocFilter.java
 *
 * Implements a FileFilter which accepts only WoW TOC files.
 *
 * Created on March 2, 2007, 4:02 PM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;
import java.io.File;
import java.io.FileFilter;
/**
 * Implements a FileFilter which accepts only WoW TOC files.
 * @author m.delbuono
 */
public class TocFilter implements FileFilter {
    
    /** Creates a new instance of TocFilter */
    public TocFilter() {
    }
    
    /** Determines if this file is a TOC file
     * @param f the <tt>File</tt> to test
     * @return <tt>true</tt> if this file is a TOC file, <tt>false</tt> if not.
     */
    public boolean accept(File f)
    {
        // TOC files must be a .toc extension
        if (!f.isFile())
            return false;
        
        String s = f.getName();
        s = s.substring(s.length()-4);      // Get last 4 chars
        if (0 == s.compareToIgnoreCase(".toc"))
            return true;
        else
            return false;
    }
    
    
    
}
