/*
 * UpdaterFrame.java
 * 
 * Shows a frame which indicates all unregistered addons, which will not be
 * updated. These addons will not be updated until the user registers them
 * with the program, at which time a WID file will be made for them, with
 * the single line $$REGISTER, which indicates the program should attempt
 * to update it on the next pass.
 *
 * Created on March 1, 2007, 12:44 PM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 * 
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;

/**
 * Shows a frame which indicates all unregistered addons, which will not be
 * updated. These addons will not be updated until the user registers them
 * with the program, at which time a WID file will be made for them, with
 * the single line $$REGISTER, which indicates the program should attempt
 * to update it on the next pass.
 *
 * @author  m.delbuono
 */
import javax.swing.*;
import java.util.Vector;
import java.io.*;
public class UpdaterFrame extends javax.swing.JFrame {
    
    /** Creates new form UpdaterFrame */
    public UpdaterFrame() {
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jFrame1 = new javax.swing.JFrame();
        buttonGroup1 = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane4 = new javax.swing.JScrollPane();
        buttonAdd = new javax.swing.JButton();
        buttonRemove = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        buttonOK = new javax.swing.JButton();
        buttonCancel = new javax.swing.JButton();

        org.jdesktop.layout.GroupLayout jFrame1Layout = new org.jdesktop.layout.GroupLayout(jFrame1.getContentPane());
        jFrame1.getContentPane().setLayout(jFrame1Layout);
        jFrame1Layout.setHorizontalGroup(
            jFrame1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
        );
        jFrame1Layout.setVerticalGroup(
            jFrame1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
        );

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Register Addons...");
        setLocationByPlatform(true);
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Unknown Addons"));
        jScrollPane3.setViewportView(unknownAddons);
        unknownAddonsList = new Vector<File>();
        unknownAddonsNameList = new Vector<String>();
        unknownAddons.setListData(unknownAddonsNameList);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 203, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("WoWInterface Addons"));
        jScrollPane4.setViewportView(wowiAddons);
        wowiAddonsList = new Vector<File>();
        wowiAddonsNameList = new Vector<String>();
        wowiAddons.setListData(wowiAddonsNameList);

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 203, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
        );
        jPanel2.getAccessibleContext().setAccessibleName("Known Addons");

        buttonAdd.setText(">>");
        buttonAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonAddActionPerformed(evt);
            }
        });

        buttonRemove.setText("<<");
        buttonRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonRemoveActionPerformed(evt);
            }
        });

        jLabel1.setText("Please choose the addons you have which are available at WoWInterface.");
        jLabel1.setVerticalAlignment(javax.swing.SwingConstants.TOP);

        jLabel2.setText("WoWIUpdater will attempt to auto-locate these addons at WoWInterface, and will ask you for");

        jLabel3.setText("more information if they cannot be found.");

        buttonOK.setText("OK");
        buttonOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonOKActionPerformed(evt);
            }
        });

        buttonCancel.setText("Cancel");
        buttonCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonCancelActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2)
                    .add(jLabel3)
                    .add(layout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(buttonAdd)
                            .add(buttonRemove))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(jLabel1, 0, 0, Short.MAX_VALUE)
                        .add(40, 40, 40))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(buttonCancel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(buttonOK)))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {jPanel1, jPanel2}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .add(0, 0, 0)
                .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(83, 83, 83)
                        .add(buttonAdd)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(buttonRemove))
                    .add(layout.createSequentialGroup()
                        .add(0, 0, 0)
                        .add(jLabel3)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 18, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(buttonOK)
                    .add(buttonCancel))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void buttonOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonOKActionPerformed
        this.dispose();
        this.Register();
        wowiAddonsNameList.clear();
        wowiAddonsList.clear();
        wowiAddons.setListData(wowiAddonsNameList);
        Main.rebuild();
    }//GEN-LAST:event_buttonOKActionPerformed

    private void buttonCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonCancelActionPerformed
        this.dispose();
    }//GEN-LAST:event_buttonCancelActionPerformed

    private void buttonRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonRemoveActionPerformed
        // Move the selected item(s) over to the "unknown" addons list
        int[] indices = wowiAddons.getSelectedIndices();
        for (int i = 0; i < indices.length; i++)
        {
            // Note the -i because each time we remove 1 in a multi-list,
            // we are shifting the indices up one, meaning the next in the list
            // will now be off by 1. -i corrects this.
            unknownAddonsNameList.add(wowiAddonsNameList.get(indices[i]-i));
            unknownAddonsList.add(wowiAddonsList.get(indices[i]-i));
            unknownAddons.setListData(unknownAddonsNameList);
            wowiAddonsNameList.remove(indices[i]-i);
            wowiAddonsList.remove(indices[i]-i);
            wowiAddons.setListData(wowiAddonsNameList);
        }
    }//GEN-LAST:event_buttonRemoveActionPerformed

    private void buttonAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonAddActionPerformed
        // Move the selected item(s) over to the "WoWI" addons list
        int[] indices = unknownAddons.getSelectedIndices();
        for (int i = 0; i < indices.length; i++)
        {
            // Note the -i because each time we remove 1 in a multi-list,
            // we are shifting the indices up one, meaning the next in the list
            // will now be off by 1. -i corrects this.
            wowiAddonsNameList.add(unknownAddonsNameList.get(indices[i]-i));
            wowiAddonsList.add(unknownAddonsList.get(indices[i]-i));
            wowiAddons.setListData(wowiAddonsNameList);
            unknownAddonsNameList.remove(indices[i]-i);
            unknownAddonsList.remove(indices[i]-i);
            unknownAddons.setListData(unknownAddonsNameList);
        }
        
    }//GEN-LAST:event_buttonAddActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new UpdaterFrame().setVisible(true);
            }
        });
    }

    private JList unknownAddons = new JList();
    private JList wowiAddons = new JList();
    private Vector<File> unknownAddonsList;
    private Vector<File> wowiAddonsList;
    private Vector<String> unknownAddonsNameList;
    private Vector<String> wowiAddonsNameList;

    /** Adds this path to the unknown addons list if it has a TOC file
     * @param f the path to add
     * @return <tt>true</tt> if the path was added, <tt>false</tt> if not
     */
    public boolean AddPath(File f) {
        // Add this path to the unknown addons list if and only if it has a toc file
        // (This will (a) assert it's not a blizzard folder, and (b) assert
        // it's a valid addon
        if (!f.isDirectory())       // Also verifies existance
            return false;
        
        File[] tocs = f.listFiles(new TocFilter());  
        if (tocs.length > 0)            // There were TOC files!
        {
            unknownAddonsList.add(f);
            unknownAddonsNameList.add(f.getName());
            unknownAddons.setListData(unknownAddonsNameList);
            return true;
        }
        else
            return false;
        
    }
    
    /** Registers selected addons, which consists of (1) making a "$$REGISTER" 
     * WID for them, and then adding them to the addons list in the main frame
     */
    private void Register()
    {
        for (int i = 0; i < wowiAddonsList.size(); i++)
        {
            try
            {
                File widFile = new File(wowiAddonsList.get(i), Main.DATA_FILE);
                widFile.createNewFile();
                FileWriter writer = new FileWriter(widFile);
                writer.write("$$REGISTER");
                writer.flush();
                writer.close();
                
                //Main.f.AddAddon(new Addon(widFile));
            }
            catch (IOException e)
            {
                // Report the error; allow the user to cancel operation if
                // desired, or retry if needed
                int result = JOptionPane.showConfirmDialog(null,
                        "Unable to write an updater data file for addon "+
                            wowiAddonsList.get(i).getName() +
                            ". Confirm that you have write permissions to its "+
                            "folder. Would you like to retry?",
                        "Configuration Error",
                        JOptionPane.YES_NO_CANCEL_OPTION,
                        JOptionPane.WARNING_MESSAGE
                );
                if (result == JOptionPane.YES_OPTION)
                    i--;                                // Back up to retry
                else if (result == JOptionPane.CANCEL_OPTION)
                    return;                             // Cancel operation
            }
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton buttonAdd;
    private javax.swing.JButton buttonCancel;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JButton buttonOK;
    private javax.swing.JButton buttonRemove;
    private javax.swing.JFrame jFrame1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    // End of variables declaration//GEN-END:variables
    
}
