/*
 * WIDFilter.java
 *
 * Implements a FileFilter for handling .WID files. Does not match
 * directories or non-WID files.
 *
 * Created on April 3, 2007, 8:20 AM
 *
 * Copyright (c) 2007 Matthew P. Del Buono
 *
 * 
 *     This file is part of the WoWInterface UI Manager (WoWIUpdater)
 *
 *     WoWIUpdater is free software; you can redistribute it and/or modify 
 *     it under the terms of the GNU General Public License as published by 
 *     the Free Software Foundation; either version 2 of the License, or 
 *     (at your option) any later version.
 *
 *     WoWIUpdater is distributed in the hope that it will be useful, but 
 *     WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *     General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 *     To contact the author, please send an e-mail to
 *     Matthew.Delbuono@erau.edu or contact "Shirik" via forum private
 *     message at http://www.wowinterface.com or contact the following
 *     physical address:
 *
 *          Matthew P. Del Buono
 *          600 S Clyde Morris Blvd
 *          Mail #146379
 *          Daytona Beach, FL 32114
 *          United States of America
 */

package wowiupdater;
import java.io.File;
import java.io.FileFilter;
/**
 * Implements a FileFilter for handling .WID files. Does not match
 * directories unless specified, or non-WID files.
 * @author m.delbuono
 */
public class WIDFilter implements FileFilter {
    /** determines if directories should be considered acceptable */
    private boolean acceptDirs;
    /** Creates a new instance of WIDFilter 
     * @param acceptDirs <tt>true</tt> if directories should be matched,
     * or <tt>false</tt> if only files should match
     */
    public WIDFilter(boolean acceptDirs) {
        this.acceptDirs = acceptDirs;
    }
    
    /** Determines if the given <tt>File</tt> matches.
     * @param pathname the <tt>File</tt> to test
     * @return <tt>true</tt> if this is a match, or <tt>false</tt> if not
     */
    public boolean accept(File pathname)
    {
        if (pathname.isDirectory())
            return acceptDirs;
        
        try
        {
            String ext = pathname.getName().substring(pathname.getName().length()-4);
            if (0==ext.compareToIgnoreCase(".wid"))
                return true;
            else
                return false;
        }
        catch(java.lang.StringIndexOutOfBoundsException e)
        {
            // Occurs if the filename is too small. This will only be
            // a case where we don't want it (return false), so handle it
            return false;
        }
    }
    
}
